package com.appian.ps.plugins.sftpsmartservices;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;

@ConnectivityServices
@Order({ "serverHostName", "serverPort", "sftpUsername", "sftpPassword",
		"sftpServerFilePath", "newDirectoryName" })
public class CreateDirectoryOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger
			.getLogger(CreateDirectoryOverSFTPBasic.class);

	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerFilePath;
	private String newDirectoryName;
	
	private String STATUS_SUCCESS = "SUCCESS";
	private String STATUS_FAILED = "FAILED";
	
	private SFTPConfig config = new SFTPConfig();

	
	private String status;
	private String error;

	@Override
	public void run() throws SmartServiceException {

		int timeoutInterval = 2 * 60 * 1000;
	

		try {
			try {
				config.connectBasicAuth(sftpUsername, serverHostName, sftpPassword, serverPort, timeoutInterval);
			} catch (Exception e) {
				LOG.error("Error connecting to SFTP server", e);
				this.error=e.getMessage();
				this.status=STATUS_FAILED;
				throw createException(e, "error.sftp_conn", e.getMessage());
			}
			
			// Changing directory if exist
			if (sftpServerFilePath != null && !sftpServerFilePath.isEmpty()) {
				Path path = Paths.get(sftpServerFilePath);
				Iterator<Path> folders = path.iterator();
				while (folders.hasNext()) {
					try {
						String folderName=folders.next().toString();
						config.getSftpChannel().cd(folderName);
					} catch (Exception e) {
						LOG.error("Error changing the directory", e);
						this.error=e.getMessage();
						this.status=STATUS_FAILED;
						throw createException(e, "error.change_directory",
								e.getMessage());
					}
				}
			}
			// Creating new Directory
			if (newDirectoryName != null) {
				try {
					config.getSftpChannel().mkdir(newDirectoryName);
				} catch (Exception e) {
					LOG.error("Error creating folder in remote server", e);
					this.error=e.getMessage();
					this.status=STATUS_FAILED;
					throw createException(e, "error.create_folder",
							e.getMessage());
				}
			}
			this.status=STATUS_SUCCESS;
		} finally {
			// Close SFTP channel and session
			config.disconnect();		}
	}

	public CreateDirectoryOverSFTPBasic(ContentService cs) {
		super();
	}

	@Input(localId = "var_local_id_1", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(localId = "var_local_id_2", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue = "22")
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(localId = "var_local_id_3", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(localId = "var_local_id_4", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(localId = "var_local_id_5", required = com.appiancorp.suiteapi.process.framework.Required.OPTIONAL)
	@Name("sftpServerFilePath")
	public void setSftpServerFilePath(String val) {
		this.sftpServerFilePath = val;
	}

	@Input(localId = "var_local_id_0", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue = "NewFolder")
	@Name("newDirectoryName")
	public void setNewDirectoryName(String val) {
		this.newDirectoryName = val;
	}
	
	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}

	private SmartServiceException createException(Throwable t, String key,
			Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(
				key, args).build();
	}
}
