package com.appian.ps.plugins.sftpsmartservices;

import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;

public class FTPConfig {

	public static final Integer SCS_ERROR=0x01;
	public static final Integer CONNECTION_ERROR = 0x02; 
	public static final Integer AUTHENTICATION_ERROR = 0x03;
	public static final Integer CERTIFICATE_ERROR = 0x04;
	public static final Integer GENERAL_ERROR = 0x05; 
	public static final Integer FILE_NOT_FOUND = 0x06; 
	public static final Integer STORE_FILE = 0x07;
	public static final Integer NON_POSITIVE_REPLY = 0x08;
	public static final Integer SOCKET_TIMEOUT = 0x09;
	
	
	
	private static final Logger LOG=Logger.getLogger(FTPConfig.class);

	private final static int PORT_NUMBER=21;

	
	public static final String[] errorMessages= {"NoError", 
			"SCS.error",
			"connection.error", 
			"authentication.error",
			"cert.error",
			"general.error",
			"file.error",
			"storefile.error",
			"nonpositive.error",
			"sockettimeout.error"
	};
	
	private String serverHostname;
	private Long serverPort;
	private String username;
	private String password;
	private FTPClient ftpClient;
	
	
	public String getServerHostname() {
		return serverHostname;
	}
	public void setServerHostname(String serverHostname) {
		this.serverHostname = serverHostname;
	}
	public Long getServerPort() {
		return serverPort;
	}
	public void setServerPort(Long serverPort) {
		this.serverPort = serverPort;
	}
	public String getUsername() {
		return username;
	}
	public void setUsername(String username) {
		this.username = username;
	}
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	} 

	public  void sendFile( Document ftpDoc,String username, String password, String host, Long port, String filePath, Long documentID, Boolean binaryTransferMode, Boolean enterLocalActiveMode ) throws Exception{
	
         
         String fileLocation = ftpDoc.getInternalFilename();
         LOG.debug("Appian document URL " + fileLocation);
         
 		//Setting client connection
        this.ftpClient=null;
        FileInputStream fis = null;
        
      //check for port
      		if (port==null || port.intValue()==0)
      			port = Long.valueOf(PORT_NUMBER);
      		
      		//Remote file location
      		String writtenFileName = ftpDoc.getDisplayName();
      		
       try
       {
    	   this.ftpClient= new FTPClient();
    	   this.ftpClient.setStrictReplyParsing(false);
	 
    	   this.ftpClient.connect(host, port.intValue());
		LOG.debug("FTPClient " + this.ftpClient.toString());
		
		LOG.debug("Local active mode : " + enterLocalActiveMode);

		  //after connecting to the server set the local passive mode
		    if (enterLocalActiveMode==true){
		    	this.ftpClient.enterLocalActiveMode();
			}else{
				this.ftpClient.enterLocalPassiveMode();
			}
		    
			LOG.info("FTP Connected ");
		if(!this.ftpClient.login(username, password))
		{
			LOG.error("Login failed" + username);
			this.ftpClient.logout();
			throw new Exception("Login failed" + username);
		}
		int reply = this.ftpClient.getReplyCode();
		LOG.debug("FTP Reply code : " + reply);
		if (!FTPReply.isPositiveCompletion(reply))
		{
			LOG.error("Bad FTP reply code. Failed");
			this.ftpClient.disconnect();
			throw new Exception("Bad FTP reply code. Failed");
			
		}
		if (binaryTransferMode=true) {
			this.ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
        } else {
            // in theory this should not be necessary as servers should default to ASCII
            // but they don't all do so - see NET-500
        	this.ftpClient.setFileType(FTP.ASCII_FILE_TYPE);
        }
		
		fis = new FileInputStream(fileLocation);
		LOG.info("File to be stored "+filePath.concat("/").concat(writtenFileName));
		boolean code=this.ftpClient.storeFile(filePath.concat("/").concat(writtenFileName), fis);
		LOG.info("File stored successfully : " + code);

       }
       catch(Exception e)
       {
    	   throw e;
       }
       finally {
		if(this.ftpClient!=null)
		{
			   try {
				   this.ftpClient.logout();
				   this.ftpClient.disconnect();
			    } catch (IOException e) {
			        String message = "Could not logout";
			        LOG.info(message);
			    }
		}
		if(fis!=null)
		{
			try
			{
				fis.close();
			}
			catch(Exception e)
			{
				LOG.info("Stream cannot be closed");
			}
		}
	}
			

	}
	
	public static boolean ftpTransferRecieveFile(String server,Long port, String username, String password,String remoteFileUrl,String internalFilePath,String internalFileName, Boolean binaryTransferMode, Boolean enterLocalActiveMode,ContentService cs, Long documentId ) throws Exception {

		try {
			FTPClient ftp = new FTPClient();
			if (port == null || port == 0) {
				port = Long.valueOf(PORT_NUMBER);
			}
			ftp.setStrictReplyParsing(false);
			ftp.connect(server, port.intValue());
			LOG.debug("FTPClient " + ftp.toString());

			if (!ftp.login(username, password)) {
				LOG.error("Login failed" + username);
				ftp.logout();
				return false;
			}
			int reply = ftp.getReplyCode();
			LOG.debug("FTP Reply code : " + reply);
			if (!FTPReply.isPositiveCompletion(reply)) {
				LOG.error("Bad FTP reply code. Failed");
				ftp.disconnect();
				return false;
			}

			LOG.debug("Binary Transfer Mode : " + binaryTransferMode);

			if (binaryTransferMode = true) {
				ftp.setFileType(FTP.BINARY_FILE_TYPE);
			} else {
				// in theory this should not be necessary as servers should default to ASCII
				// but they don't all do so - see NET-500
				ftp.setFileType(FTP.ASCII_FILE_TYPE);
			}

			LOG.debug("Local active mode : " + enterLocalActiveMode);

			if (enterLocalActiveMode == true) {
				ftp.enterLocalActiveMode();
			} else {
				ftp.enterLocalPassiveMode();
			}


			try {

				OutputStream os = new BufferedOutputStream(
						new FileOutputStream(internalFilePath
								+ internalFileName));
				InputStream is = ftp.retrieveFileStream(remoteFileUrl);
				byte[] bytesArray = new byte[4096];
				int bytesRead = -1;
				while ((bytesRead = is.read(bytesArray)) != -1) {
					os.write(bytesArray, 0, bytesRead);
				}

				boolean success = ftp.completePendingCommand();

				int returncode = ftp.getReplyCode();
				LOG.debug("FTP Return Code :" + returncode);
				LOG.debug("Status :" + success);
				cs.setSizeOfDocumentVersion(documentId);
				os.close();
				is.close();
				ftp.noop();

			} catch (Exception e) {
				LOG.error("Error receiving file", e);
			
				throw e;
			}
			ftp.logout();
			ftp.disconnect();
		} catch (Exception ex) {
			LOG.error("Error while receiving file", ex);
			throw ex;
		}

		return true;
	}

	public FTPFile[] retrieveFilesInFolderOverFTPListFiles(String server, Long port, String username, String password, String filePath, Long destinationDirectory ) throws Exception{	
		 FTPFile[] files=null;
		try
		{
			LOG.debug("Starting Retrieve files from Folder using FTP ");
			this.ftpClient = new FTPClient();
			
			if (port== null || port == 0){
				port = Long.valueOf(PORT_NUMBER);
			}
			ftpClient.setStrictReplyParsing(false);
			ftpClient.connect(server, port.intValue());
			LOG.debug("FTPClient " + ftpClient.toString());

			if(!ftpClient.login(username, password))
			{
				LOG.error("Login failed" + username);
				ftpClient.logout();
			}
			int reply = ftpClient.getReplyCode();
			LOG.debug("FTP Reply code : " + reply);
			if (!FTPReply.isPositiveCompletion(reply))
			{
				LOG.error("Bad FTP reply code. Failed");
				ftpClient.disconnect();
			}		
			
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);

			ftpClient.enterLocalPassiveMode();
			
            
            files = ftpClient.listFiles(filePath);
            LOG.debug ("Total no of files : " + files.length);
           
		
		}catch(Exception e){
			LOG.error("Error retrieving file", e);
			  throw e;
		}
		return files;
	}
	
	public  void retrieveSingleFileinFolderFTP(FTPFile file, String filePath, String internalFilePath, String internalFileName)
	{
		try
		{
			LOG.debug("Starting Retrieve files from Folder using FTP ");
			FileOutputStream fo = new FileOutputStream(internalFilePath + internalFileName);
	     	LOG.debug("File Output Stream " + fo.toString());     	
	     	String remoteFileUrl = filePath +"/"+file.getName();
	     	LOG.debug("File Path for FTP Stream " + remoteFileUrl);
	     	this.ftpClient.setBufferSize(4*1024);
	     	this.ftpClient.retrieveFile(remoteFileUrl, fo);
	     	fo.flush();
			fo.close();
		}catch(Exception e){
			LOG.error("Error retrieving file", e);
			
		}
		
	}
	
	public void disconnectFTP() throws IOException
	{
		  //Disconnect(ftp)
		if(this.ftpClient != null)
		{
        LOG.debug("All done. Sending NoOp");
        this.ftpClient.noop();
        LOG.debug("Disconnecting from FTP");
        this.ftpClient.disconnect();
		}
	}
}
