package com.appian.ps.plugins.sftpsmartservices;
import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.jcraft.jsch.ChannelSftp.LsEntry;


@ConnectivityServices
@Order ({"serverHostName", "serverPort", "sftpUsername", "sftpPassword", "sftpServerRemoteDirectory"})
public class GetRemoteDirectoryContentsOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger.getLogger(GetRemoteDirectoryContentsOverSFTPBasic.class);
	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerRemoteDirectory;
	private String STATUS_SUCCESS = "SUCCESS";
	private String STATUS_FAILED = "FAILED";
	private String[] directories;
	private String[] files;
	private String status;
	private String error;
	private SFTPConfig sftpConfig = new SFTPConfig();

	@SuppressWarnings("unchecked")
	@Override
	public void run() throws SmartServiceException {
		
	    LsEntry entry = null;
	    Vector<LsEntry> contents = null;
	    ArrayList<String> directoryAL = null;
	    ArrayList<String> fileAL = null;
	    int timeoutInterval = 2 * 60 * 1000;

    try {
      // Establish a new SFTP connection using process-specific parameters
      try {
       sftpConfig.connectBasicAuth(sftpUsername, serverHostName, sftpPassword, serverPort,timeoutInterval);
      } catch (Exception e) {
    	  LOG.error("Error connecting to SFTP server", e);
      	this.status = STATUS_FAILED;
  		this.error = e.getMessage();
          throw createException(e, "error.sftp_conn", e.getMessage());

      }

      try {
        contents = sftpConfig.getSftpChannel().ls(sftpServerRemoteDirectory);
      } catch (Exception e) {
        LOG.error("Error listing remote directory contents", e);
    	this.status = STATUS_FAILED;
  		this.error = e.getMessage();
        throw createException(e, "error_sftp_remotePath", e.getMessage());
      }

      directoryAL = new ArrayList<String>();
      fileAL = new ArrayList<String>();

      directoryAL.ensureCapacity(contents.capacity());
      fileAL.ensureCapacity(contents.capacity());

      for (int i = 0; i < contents.size(); i++) {
        entry = contents.get(i);
        if (entry.getLongname().charAt(0) == 'd') {
          if (!entry.getFilename().equals(".")
              && !entry.getFilename().equals("..")) {
            directoryAL.add(entry.getFilename());
          }

        } else {
          if (!entry.getFilename().equals("..")) {
            fileAL.add(entry.getFilename());
          }
        }
      }

      directoryAL.trimToSize();
      fileAL.trimToSize();
    } finally {
      // Close SFTP channel and session
    	sftpConfig.disconnect();
    }

		directories = directoryAL.toArray(new String[directoryAL.size()]);
		files = fileAL.toArray(new String[fileAL.size()]);
		this.status=STATUS_SUCCESS;
		return;
	}

	@Input(required = Required.OPTIONAL)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpServerRemoteDirectory")
	public void setSftpServerRemoteDirectory(String val) {
		this.sftpServerRemoteDirectory = val;
	}

	@Name("directories")
	public String[] getDirectories() {
		return directories;
	}

	@Name("files")
	public String[] getFiles() {
		return files;
	}
	
	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}

	private SmartServiceException createException(Throwable t, String key, Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
	}
}