package com.appian.ps.plugins.sftpsmartservices;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;


@ConnectivityServices
@Order ({"serverHostName", "serverPort", "sftpUsername", "sftpPassword", "sftpServerFilePath", "destinationFolder", "documentName", "documentDescription"})
public class ReceiveFileOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger.getLogger(ReceiveFileOverSFTPBasic.class);
	private final ContentService contentService;

	private Long destinationFolder;
	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerFilePath;
	private String documentName;
	private String documentDescription;
	private Long doc;
	
	private SFTPConfig config = new SFTPConfig();
	
	private String STATUS_SUCCESS = "SUCCESS";
	private String STATUS_FAILED = "FAILED";
	
	private String status;
	private String error;

	@Override
	public void run() throws SmartServiceException {
		String local_filename = null;
	    String local_pathname = null;
	    Long documentId = null;
	    Document d = null;

	    int timeoutInterval = 2 * 60 * 1000; //2 minutes converted to ms

    try {
      // Establish a new SFTP connection using process-specific parameters
      try {
    	config.connectBasicAuth(sftpUsername, serverHostName, sftpPassword, serverPort,timeoutInterval);
    	  
        LOG.debug("Channel connection complete");
      } catch (Exception e) {
        LOG.error("Error connecting to SFTP server", e);
        this.status=STATUS_FAILED;
        this.error=e.getMessage();
        throw createException(e, "error.sftp_conn", e.getMessage());
      }

      // Convert user-provided pathnames for file system compatibility
      String remote_pathname = sftpServerFilePath.replace('\\', '/');
      String remote_filename = sftpServerFilePath.substring(sftpServerFilePath
          .lastIndexOf('/') + 1);
      remote_pathname = sftpServerFilePath.substring(0,
          sftpServerFilePath.lastIndexOf('/') + 1);
      
      LOG.debug("remote_pathname: "+remote_pathname);
      LOG.debug("remote_filename:"+remote_filename);

      // Set all required parameters for the Appian Document
      // If the documentname_param field is not set, set it to the
      // remote_filename variable
      // Also, check to make sure that, if the user specifies a different
      // document name, that
      // they included a file extension. If not, duplicate file extension from
      // remote_filename.
      if (documentName == null || documentName.length() == 0)
        documentName = remote_filename;
      if (documentName.lastIndexOf('.') == -1)
        documentName += remote_filename.substring(remote_filename
            .lastIndexOf('.'));
      // Set the document description if the user does not provide one
      if (documentDescription == null)
        documentDescription = "Transferred via SFTP";

      try {
        // Create the new Appian document. Then set the description and status
        // of the document.
    	LOG.debug("Creating the document destination in Appian");
        d = new Document(destinationFolder, documentName.substring(0,
            documentName.lastIndexOf('.')), documentName.substring(documentName
            .lastIndexOf('.') + 1));
        d.setDescription(documentDescription);
        d.setState(ContentConstants.STATE_PUBLISHED);

        LOG.debug("Creating the document within the contentService");
        // Attempt to create the document in Appian using the Content Service
        documentId = contentService.create(d, ContentConstants.UNIQUE_NONE);

        // Get local filepath on Appian server where physical file was created
        local_pathname = contentService.getInternalFilename(documentId)
            .replace('\\', '/');
      } catch (Exception e) {
    	   this.status=STATUS_FAILED;
           this.error=e.getMessage();
        LOG.error("Error creating Appian document", e);
        throw createException(e, "error.file_create", e.getMessage());
      }

      // Set filename and full path respectively
      local_filename = local_pathname
          .substring(local_pathname.lastIndexOf('/') + 1);
      local_pathname = local_pathname.substring(0,
          local_pathname.lastIndexOf('/') + 1);
      LOG.debug("Local Pathname: "+local_pathname+ " Local Filename: "+local_filename);

      try {
        // Check to see if local directory exists. If it doesn't try to create
        // it.
        // Then, change to local directory where ACC document's internal
        // filename is pointing
    	LOG.debug("Creating New File");
        File local_path = new File(local_pathname);
        if (!local_path.exists())
          local_path.mkdirs();
      } catch (Exception e) {
        LOG.error("Error creating Appian document", e);
        this.status=STATUS_FAILED;
        this.error=e.getMessage();
        throw createException(e, "error.file_create", e.getMessage());
      }

      try {
        // Get remote file
    	LOG.debug("Opening Input Stream for retrieving file with parameters: remote_pathname: "+ remote_pathname + " and remote_filename: "+ remote_filename);
        InputStream fileBodyIns = config.getSftpChannel().get(remote_pathname
            + remote_filename);
        LOG.debug("Input Stream Open, Attempting to get local File with parameters: local_pathname:" + local_pathname + 
        		" and local_filename:"+local_filename);
        File fh = new File(local_pathname + local_filename);
        LOG.debug("Opening output stream");
        OutputStream out = new FileOutputStream(fh);
        byte buf[] = new byte[1024];
        int len;
        LOG.debug("Commencing reading from stream and writing to file");
        // Write remote file to local disk
        while ((len = fileBodyIns.read(buf)) > 0) {
          out.write(buf, 0, len);
        }
        
        LOG.debug("File write complete - closing handles");

        // Close all file handler connections
        out.close();
        fileBodyIns.close();

        // Update the Appian document metadata with the file size
        contentService.setSizeOfDocumentVersion(documentId);
      } catch (Exception e) {
        LOG.error("Error transferring file", e);
        this.status=STATUS_FAILED;
        this.error=e.getMessage();
        throw createException(e, "error.file_transfer", e.getMessage());
      }
		this.status=STATUS_FAILED;
    } finally {

      // Close SFTP channel and session
     config.disconnect();
    }

    	this.status=STATUS_SUCCESS;
	    // Set the "newDocument" ARV to the documentID of the new document created
	    doc = documentId;
	}

	public ReceiveFileOverSFTPBasic(ContentService cs) {
		super();
		this.contentService = cs;
	}

	@Input(localId="var_local_id_0", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("destinationFolder")
	@FolderDataType
	public void setDestinationFolder(Long val) {
		this.destinationFolder = val;
	}

	@Input(localId="var_local_id_1", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(localId="var_local_id_2", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue="22")
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(localId="var_local_id_3", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(localId="var_local_id_4", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(localId="var_local_id_5", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpServerFilePath")
	public void setSftpServerFilePath(String val) {
		this.sftpServerFilePath = val;
	}

	@Input(localId="var_local_id_6", required=com.appiancorp.suiteapi.process.framework.Required.OPTIONAL)
	@Name("documentName")
	public void setDocumentName(String val) {
		this.documentName = val;
	}

	@Input(localId="var_local_id_7", required=com.appiancorp.suiteapi.process.framework.Required.OPTIONAL)
	@Name("documentDescription")
	public void setDocumentDescription(String val) {
		this.documentDescription = val;
	}

	@Name("doc")
	@DocumentDataType
	@Input(localId="var_local_id_8")
	public Long getDoc() {
		return doc;
	}

	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}
	private SmartServiceException createException(Throwable t, String key, Object... args) {
	  return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
	}
}
