package com.appian.ps.plugins.sftpsmartservices;

import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.jcraft.jsch.ChannelSftp.LsEntry;

@ConnectivityServices
@Order({ "serverHostName", "serverPort", "sftpUsername", "sftpPassword",
		"sftpServerRemoteDirectory" })
public class RemoveDirectoryOverSFTP extends AppianSmartService {
	private static final Logger LOG = Logger
			.getLogger(GetRemoteDirectoryContentsOverSFTPBasic.class);
	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerRemoteDirectory;
	
	private String STATUS_SUCCESS = "SUCCESS";
	private String STATUS_FAILED = "FAILED";
	
	private String status;
	private String error;
	
	private SFTPConfig config = new SFTPConfig();


	LsEntry entry = null;
	Vector<LsEntry> contents = null;
	ArrayList<String> directoryAL = null;
	ArrayList<String> fileAL = null;
	@Override
	public void run() throws SmartServiceException {
		

		int timeoutInterval = 2 * 60 * 1000;
		try {
			// Establish a new SFTP connection using process-specific parameters
			try {
				config.connectBasicAuth(sftpUsername, serverHostName, sftpPassword, serverPort, timeoutInterval);
			} catch (Exception e) {
				LOG.error("Error connecting to SFTP server", e);
				this.error=e.getMessage();
				this.status=STATUS_FAILED;
				throw createException(e, "error.sftp_conn", e.getMessage());
			}
			
			// Remove Directory
						if (sftpServerRemoteDirectory != null) {
							try {
								
								config.getSftpChannel().rmdir(sftpServerRemoteDirectory);
							} catch (Exception e) {
								this.error=e.getMessage();
								this.status=STATUS_FAILED;
								LOG.error("Error removing directory in remote server", e);
								throw createException(e, "error.create_folder",
										e.getMessage());
							}
						}

		} finally {
			// Close SFTP channel and session
			config.disconnect();
		}
		this.status=STATUS_SUCCESS;

	}
	
	

	@Input(required = Required.OPTIONAL)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpServerRemoteDirectory")
	public void setSftpServerRemoteDirectory(String val) {
		this.sftpServerRemoteDirectory = val;
	}
	
	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}

	private SmartServiceException createException(Throwable t, String key,
			Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(
				key, args).build();
	}
}
