package com.appian.ps.plugins.sftpsmartservices;


import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@ConnectivityServices
@Order({"scsExternalSystemKey", "usePerUserCredentials", "ServerHost", "Port", "PathToFolder", "DestinationDirectory"})
public class RetrieveFilesInFolderOverFTP extends AppianSmartService {

	private static final Logger LOG = Logger.getLogger(RetrieveFilesInFolderOverFTP.class);
	private final SmartServiceContext smartServiceCtx;
	private String scsExternalSystemKey;
	private boolean usePerUserCredentials;
	private String serverHost;
	private Long port;
	private String pathToFolder;
	private Long destinationDirectory;
	private Long[] files;
	private String[] error;
	private String status;


	private ContentService cs;
	private final SecureCredentialsStore scs;

	private List<Long> docIds = new ArrayList<Long>();
	private List<String> errorMessages = new ArrayList<String>();

	// Secure Credentials Store variables
	private SFTPConfig config = new SFTPConfig();
	private static final String KEY_USERNAME = "username";
	private static final String KEY_PASSWORD = "password";
	private static final String NO_SCS_FIELD = "missing.scs.field";
	private final String STATUS_FAILED ="FAILED";

	private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.ReceiveFileOverFTP";	
	private String result;
	
	@Override
	public void run() throws SmartServiceException {
		// Use Secure Credential Store
	    scsExternalSystemKey = config.getScsExternalSystemKey();	    
	    usePerUserCredentials = config.getUsePerUserCredentials();
		if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
			try {
				Map<String, String> credentials = usePerUserCredentials ? scs.getUserSecuredValues(scsExternalSystemKey) : scs.getSystemSecuredValues(scsExternalSystemKey);
				if (!credentials.containsKey(KEY_USERNAME)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_USERNAME, scsExternalSystemKey);
					throw new Exception(result);
				} else if (!credentials.containsKey(KEY_PASSWORD)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_PASSWORD, scsExternalSystemKey);
					throw new Exception(result);
				} 
				config.setUsername(credentials.get(KEY_USERNAME));
				config.setPassword(credentials.get(KEY_PASSWORD));
			} catch (Exception e) {
				LOG.error("Unable to retrieve credentials from SCS", e);
				this.error = new String[] { e.getMessage() };
				return;
			}
		}		
		try {
			fetchDocumentsFromFolder(serverHost, port, config.getUsername(), config.getPassword(), pathToFolder, destinationDirectory);
		} catch (Exception e) {
			// TODO Auto-generated catch block
		errorMessages.add(e.getMessage())	;
		this.status= STATUS_FAILED;
		}
		LOG.debug("Document Id : " + docIds);
		if(docIds !=null && docIds.size()>0)
			this.files = docIds.toArray(new Long[0]);
		if (errorMessages != null && errorMessages.size()>0)
			this.error = errorMessages.toArray(new String[0]);		
	}


	private void fetchDocumentsFromFolder(String server, Long port, String username, String password, String filePath, Long destinationDirectory ) throws Exception{	
		FTPConfig ftpConfig=new FTPConfig();
		try
		{
			LOG.debug("Starting Retrieve files from Folder using FTP ");
			FTPFile[] filesarray = ftpConfig.retrieveFilesInFolderOverFTPListFiles(server, port, username, password, filePath, destinationDirectory);
            if (filesarray.length==0){
            	this.status="EMPTY FOLDER OR FOLDER NOT FOUND";
            	return;
            }

            
            //check if the folder has any files.
            Boolean hasFiles=false;
            for (FTPFile ftpFile : filesarray) {
				if(ftpFile.isFile()){
					hasFiles=true;
				}
			}
            if (!hasFiles){
            	this.status="NO FILES FOUND";
            	return;
            }
            filePath = filePath.replace('\\', '/');
            LOG.debug("File Path : " + filePath);
            
           
     
            //Process the contents of the folder......
            for(FTPFile file : filesarray){
            	LOG.debug("Processing Content: " + file.getName());
            	if (file.isDirectory()){
            		LOG.debug("Content is a directory. Ignoring content.");
            	}
            	if (file.isFile()){
            		LOG.debug("Processing file : " + file.getName());
            		
            		//Get File name and file extension
            		 String fileName = file.getName();
                     String fileExtension="";
                     if (fileName.lastIndexOf('.') != -1) {
           		      int dotIndex = fileName.lastIndexOf('.');
           		      fileExtension = fileName.substring(dotIndex + 1);
           		      fileName = fileName.substring(0,dotIndex);
           		    }
                     
                     //Create a document in Appian CMS to store the contents.
                   Document d = new Document(destinationDirectory, fileName, fileExtension);
         	       d.setState(ContentConstants.STATE_PUBLISHED);
         	       Long documentId = cs.create(d, ContentConstants.UNIQUE_NONE);
         	       docIds.add(documentId);

         	       //Get internal file path
         	       String internalFilePath = cs.getInternalFilename(documentId).replace('\\', '/');
         	       String internalFileName ="";
      	        
      	           LOG.debug("Internal File Path : " + internalFilePath);
      	        
      	           internalFileName = internalFilePath.substring(internalFilePath.lastIndexOf('/') + 1);
      	           internalFilePath = internalFilePath.substring(0, internalFilePath.lastIndexOf('/') + 1);
      	        
      	         try {
     	            // Get remote file(filpath,file,internalfilepath,internalfilename)
      	        	ftpConfig.retrieveSingleFileinFolderFTP(file, filePath, internalFilePath, internalFileName);
     				
     		        cs.setSizeOfDocumentVersion(documentId);
     		        
     	          } catch (Exception e) {
     	        	  this.errorMessages.add("Error storing file " + fileName + " : " + e.getMessage());
     	        	  this.status= "FAILED";
     	        	  throw e;
     	          }	
            	}          	
            
            }
            this.status="SUCCESS";
            ftpConfig.disconnectFTP();
           
          
		
		}catch(Exception e){
			LOG.error("Error retrieving file", e);
			this.errorMessages.add(e.getMessage());
			error=errorMessages.toArray(new String[errorMessages.size()]);;
			throw e;
		}
	}

	/**
	   * Gets the corresponding message from the bundle for a given return code
	   * 
	   * @param code
	   *          the code returned from the executeTask() method
	   * @param params
	   *          the parameter values to replace
	   * @return the plain-text message
	   */
	 protected String getMessageForCode(String code, Object... params) {
	   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
	     : smartServiceCtx.getUserLocale();
	 
	   try {
	     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

	     return MessageFormat.format(bundle.getString(code), params);
	   } catch (Exception e) {
	     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
	     return "";
	   }
	 }	
	 
	public RetrieveFilesInFolderOverFTP(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
		super();
		this.scs = scs;
		this.smartServiceCtx = smartServiceCtx;
		this.cs = cs;
	}

	@Input(required = Required.ALWAYS)
	@Name("ServerHost")
	public void setServerHost(String val) {
		this.serverHost = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("Port")
	public void setPort(Long val) {
		this.port = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("scsExternalSystemKey")
	public void setExternalSystemKey(String scsExternalSystemKey) {
	   config.setScsExternalSystemKey(scsExternalSystemKey);
	}

	@Input(required = Required.ALWAYS, defaultValue = "false")
	@Name("usePerUserCredentials")
	public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
	   config.setUsePerUserCredentials(usePerUserCredentials);
	}

	@Input(required = Required.ALWAYS)
	@Name("PathToFolder")
	public void setPathToFolder(String val) {
		this.pathToFolder = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("DestinationDirectory")
	@FolderDataType
	public void setDestinationDirectory(Long val) {
		this.destinationDirectory = val;
	}

	@Name("files")
	@DocumentDataType
	public Long[] getFiles() {
		return files;
	}

	@Name("Error")
	public String[] getError() {
		return error;
	}

	@Name("status")
	public String getStatus() {
		return status;
	}
}
