package com.appian.ps.plugins.sftpsmartservices;

import java.io.FileOutputStream;
import java.net.SocketTimeoutException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import java.util.ResourceBundle;

import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.net.ftp.FTPSClient;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@ConnectivityServices
@Order({"scsExternalSystemKey", "usePerUserCredentials", "ServerHost", "Port", "PathToFolder", "DestinationDirectory", "privateKey"})
public class RetrieveFilesInFolderOverFTPS extends AppianSmartService {

	private static final Logger LOG = Logger.getLogger(RetrieveFilesInFolderOverFTPS.class);
	private final SmartServiceContext smartServiceCtx;

	private String serverHost;
	private Long port;
	private String pathToFolder;
	private Long destinationDirectory;
	
	/* Outputs */ 
	private Long[] files;
	private String[] errorMessage;
	private Boolean isSuccessful = true; 
	private int errorCode; 

	private ContentService cs;
	private final SecureCredentialsStore scs;
	private static final int TIME_OUT = 15*60*1000; /* Milliseconds */ 
	private static final int KEEP_ALIVE = 20; /* Seconds */ 

	private List<Long> docIds = new ArrayList<Long>();
	// Secure Credentials Store variables
	private FTPSConfig config = new FTPSConfig();
	private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.RetrieveFilesInFolderOverFTPS";	

	

	
	@Override
	public void run() throws SmartServiceException {
			LOG.debug("Attempting to access secure credential store for key: "+config.getScsExternalSystemKey());
		
		try { 
			config.getAuthenticationInformation(cs,scs);
		} catch (Exception e) { 
			this.setErrorMessageForCode(FTPSConfig.SCS_ERROR, e);
			LOG.error("Access to Secure Credential Store Failed:",e); 
			return; 
		}
		
		
		fetchDocumentsFromFolder(serverHost, port, config.getUsername(), config.getPassword(), pathToFolder, destinationDirectory);
		LOG.debug("Document Id : " + docIds);
		if(docIds !=null && docIds.size()>0)
			this.files = docIds.toArray(new Long[0]);
		if(!isSuccessful) { return; }  
		
	}


	private void fetchDocumentsFromFolder(String server, Long port, String username, String password, String filePath, Long destinationDirectory ){	
		
		FTPSClient ftps = null; 

		try
		{
			LOG.debug("Starting Retrieve files from Folder using FTP ");
			ftps = config.setupConnection(cs); 
			
			config.connect(ftps);
			ftps.setSoTimeout(TIME_OUT);
			ftps.setControlKeepAliveTimeout(KEEP_ALIVE);
		} catch (Exception e) {
			this.setErrorMessageForCode(FTPSConfig.CONNECTION_ERROR, e, config.getServerHostname());
			LOG.error("Error connecting to the server", e);
			return;
		}
		
	
		try { 
            
			FTPFile[] files = ftps.listFiles(filePath);
            int reply = ftps.getReplyCode(); 
		    if (!FTPReply.isPositiveCompletion(reply)) {
		    	  this.setErrorMessageForCode(FTPSConfig.NON_POSITIVE_REPLY, null, reply);
		    	  LOG.debug("non positive reply for list files. Code returned: "+reply);
		    	  ftps.disconnect();
		    	  return;
		    }
            LOG.debug ("Total no of files : " + files.length);
            if (files.length==0){
            	this.setErrorMessageForCode(FTPSConfig.FILE_NOT_FOUND, null);
            	return;
            }

            //check if the folder has any files.
            Boolean hasFiles=false;
            for (FTPFile ftpFile : files) {
				if(ftpFile.isFile()){
					hasFiles=true;
				}
			}
            if (!hasFiles){
            	this.setErrorMessageForCode(FTPSConfig.FILE_NOT_FOUND, null);
            	ftps.disconnect();
            	return;
            }
            filePath = filePath.replace('\\', '/');
            LOG.debug("File Path : " + filePath);
            
           
     
            //Process the contents of the folder......
            for(FTPFile file : files){
            	LOG.debug("Processing Content: " + file.getName());
            	if (file.isDirectory()){
            		LOG.debug("Content is a directory. Ignoring content.");
            	}
            	if (file.isFile()){
            		LOG.debug("Processing file : " + file.getName());
            		
            		//Get File name and file extension
            		 String fileName = file.getName();
                     String fileExtension="";
                     if (fileName.lastIndexOf('.') != -1) {
           		      int dotIndex = fileName.lastIndexOf('.');
           		      fileExtension = fileName.substring(dotIndex + 1);
           		      fileName = fileName.substring(0,dotIndex);
           		    }
                     
                     //Create a document in Appian CMS to store the contents.
                   Document d = new Document(destinationDirectory, fileName, fileExtension);
         	       d.setState(ContentConstants.STATE_PUBLISHED);
         	       Long documentId = cs.create(d, ContentConstants.UNIQUE_NONE);
         	       docIds.add(documentId);

         	       //Get internal file path
         	       String internalFilePath = cs.getInternalFilename(documentId).replace('\\', '/');
         	       String internalFileName ="";
      	        
      	           LOG.debug("Internal File Path : " + internalFilePath);
      	        
      	           internalFileName = internalFilePath.substring(internalFilePath.lastIndexOf('/') + 1);
      	           internalFilePath = internalFilePath.substring(0, internalFilePath.lastIndexOf('/') + 1);
      	        
      	         try {
     	            // Get remote file
     	        	FileOutputStream fo = new FileOutputStream(internalFilePath + internalFileName);
     	        	LOG.debug("File Output Stream " + fo.toString());
     	        	
     	        	String remoteFileUrl = filePath +"/"+file.getName();
     	        	LOG.debug("File Path for FTP Stream " + remoteFileUrl);
     	        	ftps.setBufferSize(4*1024);
     	        	ftps.retrieveFile(remoteFileUrl, fo);
     	        	fo.flush();
     				fo.close();
     				
     				
     		        cs.setSizeOfDocumentVersion(documentId);
     		        
     	          } catch (Exception e) {
     	        	  this.setErrorMessageForCode(FTPSConfig.STORE_FILE, e, fileName);
     	        	  return; 
     	          }	
            	}          	
            
            }
            LOG.debug("All done. Sending NoOp");
            ftps.noop();
            LOG.debug("Disconnecting from FTP");
            ftps.disconnect();
		
		}
		catch(SocketTimeoutException se) {
			setErrorMessageForCode(FTPSConfig.SOCKET_TIMEOUT,se); 
			LOG.error("Socket Timoeut", se);
			
		}
		catch(Exception e){
			setErrorMessageForCode(FTPSConfig.GENERAL_ERROR,e); 
			LOG.error("Error retrieving file", e);
		}
	}

	/**
	   * Gets the corresponding message from the bundle for a given return code
	   * 
	   * @param code
	   *          the code returned from the executeTask() method
	   * @param params
	   *          the parameter values to replace
	   * @return the plain-text message
	   */
	 protected void setErrorMessageForCode(Integer code, Exception ex, Object... params) {
		 
	   errorCode = code.intValue(); 
	   isSuccessful = false; 
	   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
	     : smartServiceCtx.getUserLocale();
	 
	   try {
	     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

	     if (ex != null) { 
	    	 errorMessage = new String[]{
	     
	     MessageFormat.format(bundle.getString(FTPSConfig.errorMessages[code.intValue()]), params)
	     ,ex.getMessage() }; 
	     } else { 
	    	 errorMessage = new String[] {MessageFormat.format(bundle.getString(FTPSConfig.errorMessages[code.intValue()]), params)};
	    	
	     }
	    	 
	   } catch (Exception e) {
		 errorMessage = new String[] {"Check Log for error"}; 
	     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
	     return;
	   }
	 }	
	 
	public RetrieveFilesInFolderOverFTPS(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
		super();
		this.scs = scs;
		this.smartServiceCtx = smartServiceCtx;
		this.cs = cs;
	}

	@Input(required = Required.ALWAYS)
	@Name("ServerHost")
	public void setServerHost(String val) {
		config.setServerHostname(val);
	}

	@Input(required = Required.OPTIONAL)
	@Name("Port")
	public void setPort(Long val) {
		config.setServerPort(val);
	}

	@Input(required = Required.ALWAYS)
	@Name("scsExternalSystemKey")
	public void setExternalSystemKey(String scsExternalSystemKey) {
	   config.setScsExternalSystemKey(scsExternalSystemKey);
	}

	@Input(required = Required.ALWAYS, defaultValue = "false")
	@Name("usePerUserCredentials")
	public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
	   config.setUsePerUserCredentials(usePerUserCredentials);
	}

	@Input(required = Required.ALWAYS)
	@Name("PathToFolder")
	public void setPathToFolder(String val) {
		this.pathToFolder = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("DestinationDirectory")
	@FolderDataType
	public void setDestinationDirectory(Long val) {
		this.destinationDirectory = val;
	}
	
	  @Input(required = Required.OPTIONAL)
	  @Name("privateKey")
	  @DocumentDataType
	  public void setPrivateKeyFile(Long privateKeyDocId) {
	     config.setPrivateKeyFileId(privateKeyDocId);
	  }
	
	/* Outputs */ 

	@Name("files")
	@DocumentDataType
	public Long[] getFiles() {
		return files;
	}


	public String[] getErrorMessage() {
		return errorMessage;
	}


	public Boolean getIsSuccessful() {
		return isSuccessful;
	}


	public int getErrorCode() {
		return errorCode;
	}

	
	
}
