package com.appian.ps.plugins.sftpsmartservices;


import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;


@ConnectivityServices
@Order({"scsExternalSystemKey", "usePerUserCredentials", "ServerHost", "Port", "RemoteDirPath", "Document","EnterLocalActiveMode","BinaryFileType"})
public class SendFileOverFTP extends AppianSmartService {

	private static final Logger LOG = Logger.getLogger(SendFileOverFTP.class);
	private final SmartServiceContext smartServiceCtx;
	private final SecureCredentialsStore scs;

	private String scsExternalSystemKey;
	private boolean usePerUserCredentials;	
	private String serverHost;
	private Long port;
	
	private String remoteDirPath;
	private Long document;
	private String status;
	private Boolean enterLocalActiveMode=false;
	private String error;
	private Boolean binaryFileType=true;;

	private ContentService contentService;
	private final String STATUS_FAILED ="FAILED";
	private final String STATUS_SUCCESS = "SUCCESS";

	// Secure Credentials Store variables
	private SFTPConfig config = new SFTPConfig();
	private static final String KEY_USERNAME = "username";
	private static final String KEY_PASSWORD = "password";
	private static final String NO_SCS_FIELD = "missing.scs.field";
	private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.SendFileOverFTPV2";	
	private String result;
	
	@Override
	public void run() throws SmartServiceException {
		// Use Secure Credential Store
	    scsExternalSystemKey = config.getScsExternalSystemKey();	    
	    usePerUserCredentials = config.getUsePerUserCredentials();
		if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
			try {
				Map<String, String> credentials = usePerUserCredentials ? scs.getUserSecuredValues(scsExternalSystemKey) : scs.getSystemSecuredValues(scsExternalSystemKey);
				if (!credentials.containsKey(KEY_USERNAME)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_USERNAME, scsExternalSystemKey);
					throw new Exception(result);
				} else if (!credentials.containsKey(KEY_PASSWORD)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_PASSWORD, scsExternalSystemKey);
					throw new Exception(result);
				} 
				config.setUsername(credentials.get(KEY_USERNAME));
				config.setPassword(credentials.get(KEY_PASSWORD));
			} catch (Exception e) {
				LOG.error("Unable to retrieve credentials from SCS", e);
				this.error = e.getMessage();
				this.status = STATUS_FAILED;
				return;
			}
		}	
		
		try {
			sendFile(config.getUsername(), config.getPassword(), serverHost, port, remoteDirPath, document,binaryFileType,enterLocalActiveMode);
			this.status = STATUS_SUCCESS;
		} catch (Exception e) {
			LOG.error("Error sending file", e);
			this.error = e.getMessage();
			this.status= STATUS_FAILED;
		}
	}

	public SendFileOverFTP(SmartServiceContext smartServiceCtx, ContentService cs, SecureCredentialsStore scs) {
		super();
		this.scs = scs;
		this.smartServiceCtx = smartServiceCtx;
		this.contentService = cs;
	}

	/**
	 * Method that does the file transfer using PURE Java API.  
	 * @param username
	 * @param password
	 * @param host
	 * @param port
	 * @param filePath
	 * @param fileName
	 * @param documentID
	 * @throws Exception
	 */
	
	
	private void sendFile(String username, String password, String host, Long port, String filePath, Long documentID, Boolean binaryTransferMode, Boolean enterLocalActiveMode ) throws Exception{
		Document d[] = (Document[])null;
		
		 try
         {
             d = contentService.download(documentID, ContentConstants.VERSION_CURRENT, Boolean.valueOf(false));
             LOG.debug("Appian document Id " + d);
         }
         catch(Exception e)
         {
             throw e;
         }
         Document ftpDoc = d[0];
         FTPConfig ftpConfig=new FTPConfig();
         ftpConfig.sendFile(ftpDoc, username,  password,  host,  port,  filePath,  documentID,  binaryTransferMode,  enterLocalActiveMode);
	
			

	}
	 /**
	   * Gets the corresponding message from the bundle for a given return code
	   * 
	   * @param code
	   *          the code returned from the executeTask() method
	   * @param params
	   *          the parameter values to replace
	   * @return the plain-text message
	   */
	 protected String getMessageForCode(String code, Object... params) {
	   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
	     : smartServiceCtx.getUserLocale();
	 
	   try {
	     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

	     return MessageFormat.format(bundle.getString(code), params);
	   } catch (Exception e) {
	     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
	     return "";
	   }
	 }	
	 
	@Input(required = Required.ALWAYS)
	@Name("ServerHost")
	public void setServerHost(String val) {
		this.serverHost = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("Port")
	public void setPort(Long val) {
		this.port = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("scsExternalSystemKey")
	public void setExternalSystemKey(String scsExternalSystemKey) {
	   config.setScsExternalSystemKey(scsExternalSystemKey);
	}

	@Input(required = Required.ALWAYS, defaultValue = "false")
	@Name("usePerUserCredentials")
	public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
	   config.setUsePerUserCredentials(usePerUserCredentials);
	}
	
	@Input(required = Required.OPTIONAL)
	@Name("RemoteDirPath")
	public void setRemoteDirPath(String val) {
		this.remoteDirPath = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("Document")
	@DocumentDataType
	public void setDocument(Long val) {
		this.document = val;
	}

	@Name("Status")
	public String getStatus() {
		return status;
	}
	
	@Input(required = Required.OPTIONAL, defaultValue = "false")
	@Name("EnterLocalActiveMode")
	public void setEnterLocalActiveMode(Boolean val) {
		this.enterLocalActiveMode = val;
	}
	
	@Input(required = Required.OPTIONAL, defaultValue = "true")
	@Name("BinaryFileType")
	public void setBinaryFileType(Boolean val) {
		this.binaryFileType = val;
	}
	
	@Name("Error")
	public String getError() {
		return error;
	}

	
	
}

