package com.appian.ps.plugins.sftpsmartservices;

import java.io.FileInputStream;
import java.io.InputStream;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;


@ConnectivityServices
@Order ({"serverHostName", "serverPort", "sftpUsername", "sftpPassword", "sftpServerFilePath", "sftpServerFileName", "documentToCopy"})
public class SendFileOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger.getLogger(SendFileOverSFTPBasic.class);
	private final ContentService contentService;

	private Long documentToCopy;
	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerFilePath;
	private String sftpServerFileName;
	private String STATUS_SUCCESS = "SUCCESS";
	private String STATUS_FAILED = "FAILED";

	private SFTPConfig config = new SFTPConfig();

	
	private String status;
	private String error;
	@Override
	public void run() throws SmartServiceException {

		String local_filename = null;
	    String local_pathname = null;
	    String remote_filename = null;

	    int timeoutInterval = 2 * 60 * 1000;
	    Document d = null;


	    try {
	      // Obtain the current version Document object of the ACC document specified
	      d = contentService.download(documentToCopy, ContentConstants.VERSION_CURRENT, false)[0];
	    } catch (Exception e) {
	      LOG.error("Error getting current document", e);
	  	this.error=e.getMessage();
		this.status=STATUS_FAILED;
	      // The documentID specified does not exist in the Appian Collaboration Center
	      throw createException(e, "error.doc_no_exist", e.getMessage());
	    }

	    // Get the local file and path information for the specified document
	    local_pathname = d.getInternalFilename().replace('\\','/');
	    local_filename = local_pathname.substring(local_pathname.lastIndexOf('/')+1);
	    local_pathname = local_pathname.substring(0, local_pathname.lastIndexOf('/')+1);

	    // Set the name of the remote file to transfer
	    remote_filename = d.getName() + "." + d.getExtension();
	    if(sftpServerFileName.length() > 0) {
	      remote_filename = sftpServerFileName;
	    }

	    // Ensure that the remote file path terminates with a forward slash
	    if(sftpServerFilePath.charAt(sftpServerFilePath.length()-1) != '/') {
	      sftpServerFilePath = sftpServerFilePath + '/';
	    }

    try {
      try {
        // Establish a new SFTP connection using process-specific parameters
        config.connectBasicAuth(sftpUsername, serverHostName, sftpPassword, serverPort, timeoutInterval);
      } catch (Exception e) {
    		this.error=e.getMessage();
			this.status=STATUS_FAILED;
        LOG.error("Error connecting to SFTP server", e);
        throw createException(e, "error.sftp_conn", e.getMessage());
      }

      try {
        // Create a copy of the Appian document on the remote server
        InputStream fileBodyIns = new FileInputStream(local_pathname
            + local_filename);
        config.getSftpChannel().put(fileBodyIns, sftpServerFilePath + remote_filename);
      } catch (Exception e) {
        LOG.error("Error transferring file", e);
    	this.error=e.getMessage();
		this.status=STATUS_FAILED;
        throw createException(e, "error.file_transfer", e.getMessage());
      }
    } finally {
      // Close SFTP channel and session
    config.disconnect();
    }
    this.status=STATUS_SUCCESS;
	}

	public SendFileOverSFTPBasic(ContentService cs) {
		super();
		this.contentService = cs;
	}

	@Input(localId="var_local_id_0", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("documentToCopy")
	@DocumentDataType
	public void setDocumentToCopy(Long val) {
		this.documentToCopy = val;
	}

	@Input(localId="var_local_id_1", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(localId="var_local_id_2", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue="22")
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(localId="var_local_id_3", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(localId="var_local_id_4", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(localId="var_local_id_5", required=com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpServerFilePath")
	public void setSftpServerFilePath(String val) {
		this.sftpServerFilePath = val;
	}

	@Input(localId="var_local_id_6", required=com.appiancorp.suiteapi.process.framework.Required.OPTIONAL)
	@Name("sftpServerFileName")
	public void setSftpServerFileName(String val) {
		this.sftpServerFileName = val;
	}

	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}
	
	private SmartServiceException createException(Throwable t, String key, Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
	}
}
