package com.appian.ps.plugins.sftpsmartservices.old;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

@Deprecated
@PaletteInfo(paletteCategory="Deprecated Services", palette="Connectivity Services")
@Order({ "serverHostName", "serverPort", "sftpUsername", "sftpPassword",
		"sftpServerFilePath", "newDirectoryName" })
public class CreateDirectoryOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger
			.getLogger(CreateDirectoryOverSFTPBasic.class);

	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerFilePath;
	private String newDirectoryName;

	@Override
	public void run() throws SmartServiceException {

		JSch jsch = null;
		Session session = null;
		ChannelSftp sftpChannel = null;

		try {
			try {
				// Establish a new SFTP connection using process-specific
				// parameters
				jsch = new JSch();
				session = jsch.getSession(sftpUsername, serverHostName);
				session.setPassword(sftpPassword);
				session.setPort(serverPort.intValue());
				session.setConfig("StrictHostKeyChecking", "no");

				// Default Configs / Supported Ciphers can be found here:
				// https://github.com/is/jsch/blob/b2ac6d9c95a688b22cbc3316ccf3315ba2fb6f86/src/com/jcraft/jsch/JSch.java
				// session.setConfig("cipher.s2c",
				// "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");
				// session.setConfig("cipher.c2s",
				// "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");

				session.setConfig("compression.s2c",
						"zlib,zlib@openssh.com,none");
				session.setConfig("compression.c2s",
						"zlib,zlib@openssh.com,none");

				session.connect();

				sftpChannel = (ChannelSftp) session.openChannel("sftp");
				sftpChannel.connect();
			} catch (Exception e) {
				LOG.error("Error connecting to SFTP server", e);
				throw createException(e, "error.sftp_conn", e.getMessage());
			}
			
			// Changing directory if exist
			if (sftpServerFilePath != null && !sftpServerFilePath.isEmpty()) {
				Path path = Paths.get(sftpServerFilePath);
				Iterator<Path> folders = path.iterator();
				while (folders.hasNext()) {
					try {
						sftpChannel.cd(folders.next().toString());
					} catch (Exception e) {
						LOG.error("Error changing the directory", e);
						throw createException(e, "error.change_directory",
								e.getMessage());
					}
				}
			}
			// Creating new Directory
			if (newDirectoryName != null) {
				try {
					sftpChannel.mkdir(newDirectoryName);
				} catch (Exception e) {
					LOG.error("Error creating folder in remote server", e);
					throw createException(e, "error.create_folder",
							e.getMessage());
				}
			}
		} finally {
			// Close SFTP channel and session
			if (sftpChannel != null) {
				sftpChannel.disconnect();
			}
			if (session != null) {
				session.disconnect();
			}
		}
	}

	public CreateDirectoryOverSFTPBasic(ContentService cs) {
		super();
	}

	@Input(localId = "var_local_id_1", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(localId = "var_local_id_2", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue = "22")
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(localId = "var_local_id_3", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(localId = "var_local_id_4", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(localId = "var_local_id_5", required = com.appiancorp.suiteapi.process.framework.Required.OPTIONAL)
	@Name("sftpServerFilePath")
	public void setSftpServerFilePath(String val) {
		this.sftpServerFilePath = val;
	}

	@Input(localId = "var_local_id_0", required = com.appiancorp.suiteapi.process.framework.Required.ALWAYS, defaultValue = "NewFolder")
	@Name("newDirectoryName")
	public void setNewDirectoryName(String val) {
		this.newDirectoryName = val;
	}

	private SmartServiceException createException(Throwable t, String key,
			Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(
				key, args).build();
	}
}
