package com.appian.ps.plugins.sftpsmartservices.old;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;
import com.jcraft.jsch.ChannelSftp.LsEntry;

@Deprecated
@PaletteInfo(paletteCategory="Deprecated Services", palette="Connectivity Services")
@Order ({"scsExternalSystemKey", "usePerUserCredentials", "privateKey", "knownHosts", "serverHostName", "serverPort", "sftpServerRemoteDirectory"})
public class GetRemoteDirectoryContentsOverSFTP extends AppianSmartService {
	private static final Logger LOG = Logger.getLogger(GetRemoteDirectoryContentsOverSFTP.class);
	private final SmartServiceContext smartServiceCtx;
	private final ContentService cs;
	private final SecureCredentialsStore scs;
	// Smart Service inputs
	private String 	sftpServerRemoteDirectory;

	// Smart Service outputs
	private String[] directories;
	private String[] files;
	
	// Secure Credentials Store variables
	private SFTPConfig config = new SFTPConfig();
	private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.GetRemoteDirectoryContentsOverSFTPwithSCSUsernameAndPasswordV2";	

	public GetRemoteDirectoryContentsOverSFTP(SmartServiceContext smartServiceCtx, ContentService cs, SecureCredentialsStore scs) {
		super();
		this.cs = cs;
		this.scs = scs;
		this.smartServiceCtx = smartServiceCtx;
	}	
	@SuppressWarnings("unchecked")
	@Override
	public void run() throws SmartServiceException {
	    LsEntry entry = null;
	    Vector<LsEntry> contents = null;
	    ArrayList<String> directoryAL = null; 
	    ArrayList<String> fileAL = null;

    try {
      // Establish a new SFTP connection using process-specific parameters
      try {
        config.connect(cs, scs);
      } catch (Exception e) {
        LOG.error("Error connecting to SFTP server", e);
        throw createException(e, "error.sftp_conn", e.getMessage());
      }

      try {
        contents = config.getSftpChannel().ls(sftpServerRemoteDirectory);
      } catch (Exception e) {
        LOG.error("Error listing remote directory contents", e);
        throw createException(e, "error.remote_path", e.getMessage());
      }

      directoryAL = new ArrayList<String>();
      fileAL = new ArrayList<String>();

      directoryAL.ensureCapacity(contents.capacity());
      fileAL.ensureCapacity(contents.capacity());

      for (int i = 0; i < contents.size(); i++) {
        entry = contents.get(i);
        if (entry.getLongname().charAt(0) == 'd') {
          if (!entry.getFilename().equals(".")
              && !entry.getFilename().equals("..")) {
            directoryAL.add(entry.getFilename());
          }

        } else {
          if (!entry.getFilename().equals("..")) {
            fileAL.add(entry.getFilename());
          }
        }
      }

      directoryAL.trimToSize();
      fileAL.trimToSize();
    } finally {
        config.disconnect();
    }
		// Outputs
		directories = directoryAL.toArray(new String[directoryAL.size()]);
		files = fileAL.toArray(new String[fileAL.size()]);
		
		return;
	}

	/**
	   * Gets the corresponding message from the bundle for a given return code
	   * 
	   * @param code
	   *          the code returned from the executeTask() method
	   * @param params
	   *          the parameter values to replace
	   * @return the plain-text message
	   */
	 protected String getMessageForCode(String code, Object... params) {
	   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
	     : smartServiceCtx.getUserLocale();
	 
	   try {
	     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

	     return MessageFormat.format(bundle.getString(code), params);
	   } catch (Exception e) {
	     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
	     return "";
	   }
	 }	

	@Input(required = Required.ALWAYS)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		config.setServerHostname(val);
	}

	@Input(required = Required.ALWAYS, defaultValue = "22")
	@Name("serverPort")
	public void setServerPort(Long val) {
		config.setServerPort(val);
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpServerRemoteDirectory")
	public void setSftpServerRemoteDirectory(String val) {
		this.sftpServerRemoteDirectory = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("scsExternalSystemKey")
	public void setExternalSystemKey(String scsExternalSystemKey) {
		config.setScsExternalSystemKey(scsExternalSystemKey);
	}

	@Input(required = Required.ALWAYS, defaultValue = "false")
	@Name("usePerUserCredentials")
	public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
		config.setUsePerUserCredentials(usePerUserCredentials);
	}

	@Input(required = Required.OPTIONAL)
	@Name("privateKey")
	@DocumentDataType
	public void setPrivateKeyFile(Long privateKeyDocId) {
		config.setPrivateKeyFileId(privateKeyDocId);
	}

	@Input(required = Required.OPTIONAL)
	@Name("knownHosts")
	@DocumentDataType
	public void setKnownHostsFile(Long knownHostsDocId) {
		config.setKnownHostsFileId(knownHostsDocId);
	}

	@Name("directories")
	public String[] getDirectories() {
		return directories;
	}

	@Name("files")
	public String[] getFiles() {
		return files;
	}

	private SmartServiceException createException(Throwable t, String key, Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
	}

}