package com.appian.ps.plugins.sftpsmartservices.old;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.Approval;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.MessageContainer;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@Deprecated
@PaletteInfo(paletteCategory="Deprecated Services", palette="Connectivity Services")
@Order({"serverHostName", "serverPort", "username", "password", "remoteFilePath",
  "destinationFolder", "destinationDocument", "documentName", "documentDescription"})
public class ReceiveFileOverSCP extends AppianSmartService {

  private static final Logger LOG = Logger.getLogger(ReceiveFileOverSCP.class);
  private static final String INPUT_DESTINATION_FOLDER = "destinationFolder";
  private static final String INPUT_DESTINATION_DOCUMENT = "destinationDocument";

  private final SmartServiceContext smartServiceCtx;
  private ContentService contentService;
  private final SecureCredentialsStore scs;

  // INPUTS
  private String scsExternalSystemKey;
  private boolean usePerUserCredentials;  
  private String serverHostName;
  private Long serverPort;
  private String remoteFilePath;

  // must specify one of the following
  private Long destinationFolder;
  private Long destinationDocument;

  // optional
  private String documentName;
  private String documentDescription;

  // OUTPUTS
  private Long newDocument;
  
  // Secure Credentials Store variables
  private SFTPConfig config = new SFTPConfig();
  private static final String KEY_USERNAME = "username";
  private static final String KEY_PASSWORD = "password";
  private static final String NO_SCS_FIELD = "missing.scs.field";
  private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.ReceiveFileOverFTP";	
  private String result;
  
  @Override
  public void run() throws SmartServiceException {

		// Use Secure Credential Store
	    scsExternalSystemKey = config.getScsExternalSystemKey();	    
	    usePerUserCredentials = config.getUsePerUserCredentials();
		if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
			try {
				Map<String, String> credentials = usePerUserCredentials ? scs.getUserSecuredValues(scsExternalSystemKey) : scs.getSystemSecuredValues(scsExternalSystemKey);
				if (!credentials.containsKey(KEY_USERNAME)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_USERNAME, scsExternalSystemKey);
					throw new Exception(result);
				} else if (!credentials.containsKey(KEY_PASSWORD)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_PASSWORD, scsExternalSystemKey);
					throw new Exception(result);
				} 
				config.setUsername(credentials.get(KEY_USERNAME));
				config.setPassword(credentials.get(KEY_PASSWORD));
			} catch (Exception e) {
				LOG.error("Unable to retrieve credentials from SCS", e);
				throw TransferUtils.createException(this.getClass(), e, "error.scp_conn", e.getMessage());
			}
		}	   
    if (StringUtils.isBlank(documentDescription)) {
      documentDescription = "Transferred via SCP";
    }
    // Convert user-provided pathnames for file system compatibility
    remoteFilePath = remoteFilePath.replace('\\', '/');
    String remoteFileName = remoteFilePath.substring(remoteFilePath.lastIndexOf('/') + 1);

    // get the remote extension so we can append it to the document name if it was omitted
    String remoteExtension = "";
    if (remoteFileName.lastIndexOf('.') != -1) {
      int dotIndex = remoteFileName.lastIndexOf('.');
      remoteExtension = remoteFileName.substring(dotIndex + 1);
    }
    if (StringUtils.isBlank(documentName)) {
      documentName = remoteFileName;
    }

    // separate the document name and extension for the API call
    String documentExtension = "";
    if (documentName.lastIndexOf('.') != -1) {
      int dotIndex = documentName.lastIndexOf('.');
      documentExtension = documentName.substring(dotIndex + 1);
      documentName = documentName.substring(0, dotIndex);
    }
    // if no document extension was specified but the remote file has an extension, use remote extension
    if (StringUtils.isBlank(documentExtension) && StringUtils.isNotBlank(remoteExtension)) {
      documentExtension = remoteExtension;
    } else if (!documentExtension.equalsIgnoreCase(remoteExtension)) {
      LOG.warn("Attempting to transfer file with extension '" + remoteExtension +
        "' to Document with extension '" + documentExtension + "'");
    }
    Document d = null;
    Long documentId = null;
    String internalFilePath = null;
    try {
      if (null == destinationDocument) {
        // Create the new Appian document. Then set the description and status of the document.
        d = new Document(destinationFolder, documentName, documentExtension);
        d.setDescription(documentDescription);
        d.setState(ContentConstants.STATE_PUBLISHED);
        // Attempt to create the document in Appian using the Content Service
        documentId = contentService.create(d, ContentConstants.UNIQUE_NONE);
      } else {
        Document doc = (Document) contentService.getVersion(destinationDocument,
          ContentConstants.VERSION_CURRENT);
        // prepare the document, indicating the service to assign a new file path
        // to the new version of the document
        doc.setFileSystemId(ContentConstants.ALLOCATE_FSID);
        // create a new version
        Approval a = contentService.createVersion(doc, ContentConstants.UNIQUE_NONE);
        // get the file path of the new version
        documentId = a.getId()[0];
      }

      // Get local filepath on Appian server where physical file was created
      internalFilePath = contentService.getInternalFilename(documentId).replace('\\', '/');
    } catch (Exception e) {
      LOG.error("Error creating Appian document", e);
      throw TransferUtils.createException(this.getClass(), e, "error.file_create");
    }

    TransferUtils.receiveFileOverSCP(serverHostName, serverPort, config.getUsername(), config.getPassword(), remoteFilePath,
        internalFilePath, this.getClass());
    try {
      contentService.setSizeOfDocumentVersion(documentId);
    } catch (Exception e) {
      LOG.error("Error setting document size", e);
      throw TransferUtils.createException(this.getClass(), e, "error.insufficient_permission");
    }

    // Set the "newDocument" ARV to the documentID of the new document created
    newDocument = documentId;
  }

  public ReceiveFileOverSCP(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
    super();
    this.scs = scs;
    this.smartServiceCtx = smartServiceCtx;
    this.contentService = cs;
  }

  public void validate(MessageContainer messages) {
    if (null == destinationFolder && null == destinationDocument) {
      messages.addError(INPUT_DESTINATION_FOLDER, "error.missing_folder_or_doc");
    }
    if (null != destinationFolder && null != destinationDocument) {
      messages.addError(INPUT_DESTINATION_FOLDER, "error.only_specify_folder_or_doc");
    }
    String remoteFileName = remoteFilePath.substring(remoteFilePath.lastIndexOf('/') + 1);
    if (null != destinationDocument && StringUtils.isNotBlank(remoteFileName)) {
      try {
        Document d = (Document) contentService.getVersion(destinationDocument,
          ContentConstants.VERSION_CURRENT);
        String ext = d.getExtension();
        int dotIndex = remoteFileName.lastIndexOf('.');
        String remoteExtension = remoteFileName.substring(dotIndex + 1);
        if (!ext.equalsIgnoreCase(remoteExtension)) {
          messages.addError(INPUT_DESTINATION_DOCUMENT, "error.extension_mismatch");
        }
      } catch (Exception e) {
        LOG.error("Error getting current document", e);
        messages.addError(INPUT_DESTINATION_DOCUMENT, "error.insufficient_permission");
      }
    }
  }

	 /**
   * Gets the corresponding message from the bundle for a given return code
   * 
   * @param code
   *          the code returned from the executeTask() method
   * @param params
   *          the parameter values to replace
   * @return the plain-text message
   */
 protected String getMessageForCode(String code, Object... params) {
   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
     : smartServiceCtx.getUserLocale();
 
   try {
     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

     return MessageFormat.format(bundle.getString(code), params);
   } catch (Exception e) {
     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
     return "";
   }
 }	
 
  @Input(required = Required.ALWAYS)
  @Name("serverHostName")
  public void setServerHostName(String val) {
    this.serverHostName = val;
  }

  @Input(required = Required.ALWAYS, defaultValue = "22")
  @Name("serverPort")
  public void setServerPort(Long val) {
    this.serverPort = val;
  }
  
  @Input(required = Required.ALWAYS)
  @Name("scsExternalSystemKey")
  public void setExternalSystemKey(String scsExternalSystemKey) {
	config.setScsExternalSystemKey(scsExternalSystemKey);
	}
  
  @Input(required = Required.ALWAYS, defaultValue = "false")
  @Name("usePerUserCredentials")
  public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
	config.setUsePerUserCredentials(usePerUserCredentials);
	}

  @Input(required = Required.ALWAYS)
  @Name("remoteFilePath")
  public void setRemoteFilePath(String val) {
    this.remoteFilePath = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("destinationFolder")
  @FolderDataType
  public void setTargetFolder(Long val) {
    this.destinationFolder = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("destinationDocument")
  @DocumentDataType
  public void setTargetDocument(Long val) {
    this.destinationDocument = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("documentName")
  public void setDocumentName(String val) {
    this.documentName = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("documentDescription")
  public void setDocumentDescription(String val) {
    this.documentDescription = val;
  }

  @Name("newDocument")
  @DocumentDataType
  public Long getNewDocument() {
    return newDocument;
  }

}
