package com.appian.ps.plugins.sftpsmartservices.old;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@Deprecated
@PaletteInfo(paletteCategory="Deprecated Services", palette="Connectivity Services")
@Order({"serverHostName", "serverPort", "username", "password", "remoteFilePath", "sourceDocument"})
public class SendFileOverSCP extends AppianSmartService {

  private static final Logger LOG = Logger.getLogger(SendFileOverSCP.class);
  private final SmartServiceContext smartServiceCtx;
  private ContentService contentService;
  private final SecureCredentialsStore scs;

  // Secure Credentials Store variables
  private SFTPConfig config = new SFTPConfig();
  private static final String KEY_USERNAME = "username";
  private static final String KEY_PASSWORD = "password";
  private static final String NO_SCS_FIELD = "missing.scs.field";
  private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.SendFileOverSCP";	
  private String result;
  
  // Smart Service inputs
  private String serverHostName;
  private Long serverPort;
  private String scsExternalSystemKey;
  private boolean usePerUserCredentials;
  private String remoteFilePath;
  private Long sourceDocument;

  private SmartServiceException createException(Throwable t, String key, Object... args) {
    return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
  }

  @Override
  public void run() throws SmartServiceException {
	// Secure Credentials Store
    scsExternalSystemKey = config.getScsExternalSystemKey();
    usePerUserCredentials = config.getUsePerUserCredentials();
	if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
		try {
			Map<String, String> credentials = usePerUserCredentials ? scs.getUserSecuredValues(scsExternalSystemKey) : scs.getSystemSecuredValues(scsExternalSystemKey);
			if (!credentials.containsKey(KEY_USERNAME)) {	
				result = getMessageForCode(NO_SCS_FIELD, KEY_USERNAME, scsExternalSystemKey);
				throw new Exception(result);
			} else if (!credentials.containsKey(KEY_PASSWORD)) {
				result = getMessageForCode(NO_SCS_FIELD, KEY_PASSWORD, scsExternalSystemKey);
				throw new Exception(result);
			}
			config.setUsername(credentials.get(KEY_USERNAME));
			config.setPassword(credentials.get(KEY_PASSWORD));
		} catch (Exception e) {
			LOG.error("Unable to retrieve credentials from SCS", e);
			throw createException(e, "error.scp_conn", e.getMessage());
		}
	}	        	  
    String internalFilePath;
    try {
      // Obtain the current version Document object of the ACC document specified
      sourceDocument = contentService.getVersionId(sourceDocument, ContentConstants.VERSION_CURRENT);
      internalFilePath = contentService.getInternalFilename(sourceDocument).replace('\\', '/');
    } catch (Exception e) {
      LOG.error("Error getting current document", e);
      // The documentID specified does not exist in the Appian Collaboration Center
      throw createException(e, "error.doc_no_exist", e.getMessage());
    }

    TransferUtils.sendFileOverSCP(serverHostName, serverPort, config.getUsername(), config.getPassword(), remoteFilePath,
        internalFilePath, this.getClass());

  }

  public SendFileOverSCP(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
    super();
    this.scs = scs;
    this.smartServiceCtx = smartServiceCtx;
    this.contentService = cs;
  }

  /**
   * Gets the corresponding message from the bundle for a given return code
   * 
   * @param code
   *          the code returned from the executeTask() method
   * @param params
   *          the parameter values to replace
   * @return the plain-text message
   */
  protected String getMessageForCode(String code, Object... params) {
    Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
     : smartServiceCtx.getUserLocale();
 
    try {
      ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

      return MessageFormat.format(bundle.getString(code), params);
    } catch (Exception e) {
      LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
      return "";
    }
  }  
  
  @Input(required = Required.ALWAYS)
  @Name("serverHostName")
  public void setServerHostName(String val) {
    this.serverHostName = val;
  }

  @Input(required = Required.ALWAYS, defaultValue = "22")
  @Name("serverPort")
  public void setServerPort(Long portNumber) {
    this.serverPort = portNumber;
  }
  
  @Input(required = Required.ALWAYS)
  @Name("scsExternalSystemKey")
  public void setExternalSystemKey(String scsExternalSystemKey) {
     config.setScsExternalSystemKey(scsExternalSystemKey);
  }

  @Input(required = Required.ALWAYS, defaultValue = "false")
  @Name("usePerUserCredentials")
  public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
     config.setUsePerUserCredentials(usePerUserCredentials);
  }
  

  @Input(required = Required.ALWAYS)
  @Name("remoteFilePath")
  public void setRemoteFilePath(String val) {
    this.remoteFilePath = val;
  }

  @Input(required = Required.ALWAYS)
  @Name("sourceDocument")
  @DocumentDataType
  public void setSourceDocument(Long val) {
    this.sourceDocument = val;
  }

}
