// Author: Phil Russell
// Date Modified: 2010-August-19
// Appian Version: 6.0.3+

package com.appian.ps.plugins.sftpsmartservices.old;

import java.io.FileInputStream;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@Deprecated
@PaletteInfo(paletteCategory="Deprecated Services", palette="Connectivity Services")
@Order({"scsExternalSystemKey", "usePerUserCredentials", "privateKey", "knownHosts", "serverHostName", "serverPort", "remoteFilePath", "sourceDocument"})
public class SendFileOverSFTP extends AppianSmartService {
  private static final Logger LOG = Logger.getLogger(SendFileOverSFTP.class);
  private final SmartServiceContext smartServiceCtx;
  private final ContentService contentService;
  private final SecureCredentialsStore scs;

  // Secure Credentials Store variables
  private SFTPConfig config = new SFTPConfig();
  private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.SendFileOverSFTPwithSCSUsernameAndPasswordV2";	

  // Smart Service inputs
  private String remoteFilePath;
  private Long sourceDocument;

  public SendFileOverSFTP(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
    super();
    this.scs = scs;
    this.smartServiceCtx = smartServiceCtx;
    this.contentService = cs;
  }

  private SmartServiceException createException(Throwable t, String key, Object... args) {
    return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
  }

  @Input(required = Required.ALWAYS)
  @Name("scsExternalSystemKey")
  public void setExternalSystemKey(String scsExternalSystemKey) {
     config.setScsExternalSystemKey(scsExternalSystemKey);
  }
  
  @Input(required = Required.ALWAYS, defaultValue = "false")
  @Name("usePerUserCredentials")
  public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
     config.setUsePerUserCredentials(usePerUserCredentials);
  }  

  @Input(required = Required.OPTIONAL)
  @Name("privateKey")
  @DocumentDataType
  public void setPrivateKeyFile(Long privateKeyDocId) {
     config.setPrivateKeyFileId(privateKeyDocId);
  }

  @Input(required = Required.OPTIONAL)
  @Name("knownHosts")
  @DocumentDataType
  public void setKnownHostsFile(Long knownHostsDocId) {
     config.setKnownHostsFileId(knownHostsDocId);
  }

  @Name("serverHostName")
  @Input(required = Required.ALWAYS)
  public void setServerHostName(String hostName) {
    config.setServerHostname(hostName);
  }

  @Name("serverPort")
  @Input(required = Required.ALWAYS, defaultValue = "22")
  public void setServerPort(Long portNumber) {
    config.setServerPort(portNumber);
  }

  @Name("remoteFilePath")
  @Input(required = Required.ALWAYS)
  public void setRemoteFilePath(String serverFilepath) {
    this.remoteFilePath = serverFilepath;
  }

  @Name("sourceDocument")
  @Input(required = Required.ALWAYS)
  @DocumentDataType
  public void setSourceDocument(Long documentId) {
    this.sourceDocument = documentId;
  }

  @Override
  public void run() throws SmartServiceException {
    String internalFilePath;
    Document d = null;
    try {
      // Obtain the current version Document object of the ACC document specified
      sourceDocument = contentService.getVersionId(sourceDocument, ContentConstants.VERSION_CURRENT);
      internalFilePath = contentService.getInternalFilename(sourceDocument).replace('\\', '/');
      // Obtain the current version Document object of the ACC document specified
      d = contentService.download(sourceDocument, ContentConstants.VERSION_CURRENT, false)[0];
    } catch (Exception e) {
      LOG.error("Error obtaining document to send path", e);

      // The documentID specified does not exist in the Appian Collaboration Center
      throw createException(e, "error.doc_no_exist", e.getMessage());
    }
    
    // Set the name of the remote file to transfer
    String remoteFilename = d.getName() + "." + d.getExtension();
    
    // Ensure that the remote file path terminates with a forward slash
    if(remoteFilePath.charAt(remoteFilePath.length()-1) != '/') {
      remoteFilePath = remoteFilePath + '/';
    }
    
    try {
      // Establish a new SFTP connection using process-specific parameters
      config.connect(contentService, scs);
    } catch (Exception e) {
      LOG.error("Error connecting to remote SFTP server", e);
      throw createException(e, "error.sftp_conn", e.getMessage());
    }

    try (InputStream fileBodyIns = new FileInputStream(internalFilePath)) {
      // Create a copy of the Appian document on the remote server
      config.getSftpChannel().put(fileBodyIns, remoteFilePath + remoteFilename);
    } catch (Exception e) {
      LOG.error("Error transferring file to SFTP server", e);
      throw createException(e, "error.file_transfer", e.getMessage());
    } finally {
      config.disconnect();
    }
  }

  /**
   * Gets the corresponding message from the bundle for a given return code
   * 
   * @param code
   *          the code returned from the executeTask() method
   * @param params
   *          the parameter values to replace
   * @return the plain-text message
   */
  protected String getMessageForCode(String code, Object... params) {
    Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
     : smartServiceCtx.getUserLocale();
 
    try {
      ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

      return MessageFormat.format(bundle.getString(code), params);
    } catch (Exception e) {
      LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
      return "";
    }
  }    
}
