package com.appian.ps.plugins.sftpsmartservices.old;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

public class TransferUtils {
  private static final Logger LOG = Logger.getLogger(TransferUtils.class);

  protected static void receiveFileOverSCP(String serverHostName, Long serverPort, String username,
    String password, String remoteFilePath, String internalFilePath, Class callingClass)
    throws SmartServiceException {
    JSch jsch = null;
    Session session = null;
    OutputStream out = null;
    InputStream in = null;
    Channel channel = null;
    try {
      // Establish a new SCP connection using process-specific parameters
      jsch = new JSch();
      session = jsch.getSession(username, serverHostName, serverPort.intValue());
      session.setPassword(password);
      session.setConfig("StrictHostKeyChecking", "no");
      session.connect();

      String command = "scp -f " + remoteFilePath;
      channel = session.openChannel("exec");
      ((ChannelExec) channel).setCommand(command);
      out = channel.getOutputStream();
      in = channel.getInputStream();

      channel.connect();
      sendAck(out);

    } catch (Exception e) {
      LOG.error("Error connecting to SCP server", e);
      throw createException(callingClass, e, "error.scp_conn", e.getMessage());
    }

    FileOutputStream fos = null;
    try {
      byte[] buf = new byte[1024];
      while (true) {
        int c = checkAck(in);
        if (c != 'C') {
          break;
        }
        // read '0644 '
        in.read(buf, 0, 5);

        long filesize = 0L;
        while (true) {
          if (in.read(buf, 0, 1) < 0) {// error
            break;
          }
          if (buf[0] == ' ') {
            break;
          }
          filesize = filesize * 10L + (long) (buf[0] - '0');
        }

        String file = null;
        for (int i = 0;; i++) {
          in.read(buf, i, 1);
          if (buf[i] == (byte) 0x0a) {
            file = new String(buf, 0, i);
            break;
          }
        }
        LOG.info("filesize=" + filesize + ", file=" + file);

        sendAck(out);

        // read a content of remote file into internal file path
        fos = new FileOutputStream(internalFilePath);
        int chunk;
        while (true) {
          if (buf.length < filesize) {
            chunk = buf.length;
          } else {
            chunk = (int) filesize;
          }
          chunk = in.read(buf, 0, chunk);
          if (chunk < 0) { // error
            break;
          }
          fos.write(buf, 0, chunk);
          filesize -= chunk;
          if (filesize == 0L) {
            break;
          }
        }
        if (checkAck(in) != 0) {
          throw new IllegalStateException("checkAck() returned non 0 after writing file");
        }
        sendAck(out);
      }
      LOG.info("Completed downloading " + remoteFilePath + " from " + serverHostName + " to " +
        internalFilePath);
    } catch (Exception e) {
      LOG.error("Error transferring file", e);
      throw createException(callingClass, e, "error.scp_transfer", e.getMessage());
    } finally {
      try {
        if (out != null) {
          out.close();
        }
        if (in != null) {
          in.close();
        }
        if (fos != null) {
          fos.close();
          fos = null;
        }
        if (null != channel) {
          channel.disconnect();
        }
        if (null != session) {
          session.disconnect();
        }
      } catch (Exception e) {
        LOG.error("Error transferring file", e);
        throw createException(callingClass, e, "error.scp_transfer", e.getMessage());
      }
    }
  }

  protected static void sendFileOverSCP(String serverHostName, Long serverPort, String username,
    String password, String remoteFilePath, String internalFilePath, Class callingClass)
    throws SmartServiceException {

    JSch jsch = null;
    Session session = null;
    FileInputStream fis = null;
    OutputStream out = null;
    InputStream in = null;
    Channel channel = null;
    try {
      // Establish a new SCP connection using process-specific parameters
      jsch = new JSch();
      session = jsch.getSession(username, serverHostName, serverPort.intValue());
      session.setPassword(password);
      session.setConfig("StrictHostKeyChecking", "no");
      session.connect();

      String command = "scp -p -t " + remoteFilePath;
      channel = session.openChannel("exec");
      ((ChannelExec) channel).setCommand(command);

      out = channel.getOutputStream();
      in = channel.getInputStream();
      channel.connect();
      if (checkAck(in) != 0) {
        throw new IllegalStateException("checkAck() returned non 0 after channel connect");
      }
    } catch (Exception e) {
      LOG.error("Error connecting to SCP server", e);
      throw createException(callingClass, e, "error.scp_conn", e.getMessage());
    }

    try {
      // send "C0644 filesize filename", where filename should not include '/'
      // see http://blogs.sun.com/janp/entry/how_the_scp_protocol_works
      String remoteFileName = remoteFilePath.substring(remoteFilePath.lastIndexOf("/") + 1);
      long filesize = (new File(internalFilePath)).length();
      String command = "C0644 " + filesize + " " + remoteFileName + "\n";
      out.write(command.getBytes());
      out.flush();
      if (checkAck(in) != 0) {
        throw new IllegalStateException("checkAck() returned non 0 after protocol setup");
      }
      // send a contents of collab document
      fis = new FileInputStream(internalFilePath);
      byte[] buf = new byte[1024];
      while (true) {
        int len = fis.read(buf, 0, buf.length);
        if (len <= 0)
          break;
        out.write(buf, 0, len);
      }
      sendAck(out);
      if (checkAck(in) != 0) {
        throw new IllegalStateException("checkAck() returned non 0 after transfer");
      }
      LOG.info("Completed uploading " + internalFilePath + " to " + remoteFilePath + " on " + serverHostName);

    } catch (Exception e) {
      LOG.error("Error transferring file", e);
      throw createException(callingClass, e, "error.scp_transfer", e.getMessage());
    } finally {
      try {
        if (null != fis) {
          fis.close();
        }
        if (out != null) {
          out.close();
        }
        if (null != channel) {
          channel.disconnect();
        }
        if (null != session) {
          session.disconnect();
        }
      } catch (Exception e) {
        LOG.error("Error transferring file", e);
        throw createException(callingClass, e, "error.scp_transfer", e.getMessage());
      }
    }
  }

  static int checkAck(InputStream in) throws IOException {
    int b = in.read();
    // b may be 0 for success,
    // 1 for error,
    // 2 for fatal error,
    // -1
    if (b == 0)
      return b;
    if (b == -1)
      return b;

    if (b == 1 || b == 2) {
      StringBuffer sb = new StringBuffer();
      int c;
      do {
        c = in.read();
        sb.append((char) c);
      } while (c != '\n');
      if (b == 1) { // error
        LOG.error(sb.toString());
      }
      if (b == 2) { // fatal error
        LOG.error(sb.toString());
      }
    }
    return b;
  }

  static void sendAck(OutputStream out) throws IOException {
    // send '\0'
    out.write(0);
    out.flush();
  }

  protected static SmartServiceException createException(Class smartServiceClass, Throwable t, String key,
    Object... args) {
    return new SmartServiceException.Builder(smartServiceClass, t).userMessage(key, args).build();
  }
}
