package com.appian.ps.plugins.sftpsmartservices;
import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.ChannelSftp.LsEntry;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

@ConnectivityServices
@Order ({"serverHostName", "serverPort", "sftpUsername", "sftpPassword", "sftpServerRemoteDirectory"})
public class GetRemoteDirectoryContentsOverSFTPBasic extends AppianSmartService {
	private static final Logger LOG = Logger.getLogger(GetRemoteDirectoryContentsOverSFTPBasic.class);
	private String serverHostName;
	private Long serverPort;
	private String sftpUsername;
	private String sftpPassword;
	private String sftpServerRemoteDirectory;
	private String[] directories;
	private String[] files;

	@SuppressWarnings("unchecked")
	@Override
	public void run() throws SmartServiceException {
		JSch jsch = null;
	    Session session = null;
	    ChannelSftp sftpChannel = null;
	    LsEntry entry = null;
	    Vector<LsEntry> contents = null;
	    ArrayList<String> directoryAL = null;
	    ArrayList<String> fileAL = null;

    try {
      // Establish a new SFTP connection using process-specific parameters
      try {
        jsch = new JSch();
        session = jsch.getSession(sftpUsername, serverHostName);
        session.setPassword(sftpPassword);
        session.setPort(serverPort.intValue());
        session.setConfig("StrictHostKeyChecking", "no");
        session
            .setConfig(
                "cipher.s2c",
                "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");
        session
            .setConfig(
                "cipher.c2s",
                "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");
        session.connect();

        sftpChannel = (ChannelSftp) session.openChannel("sftp");
        sftpChannel.connect();
      } catch (Exception e) {
        LOG.error("Error connecting to SFTP server", e);
        throw createException(e, "error.sftp_conn", e.getMessage());
      }

      try {
        contents = sftpChannel.ls(sftpServerRemoteDirectory);
      } catch (Exception e) {
        LOG.error("Error listing remote directory contents", e);
        throw createException(e, "error_sftp_remotePath", e.getMessage());
      }

      directoryAL = new ArrayList<String>();
      fileAL = new ArrayList<String>();

      directoryAL.ensureCapacity(contents.capacity());
      fileAL.ensureCapacity(contents.capacity());

      for (int i = 0; i < contents.size(); i++) {
        entry = contents.get(i);
        if (entry.getLongname().charAt(0) == 'd') {
          if (!entry.getFilename().equals(".")
              && !entry.getFilename().equals("..")) {
            directoryAL.add(entry.getFilename());
          }

        } else {
          if (!entry.getFilename().equals("..")) {
            fileAL.add(entry.getFilename());
          }
        }
      }

      directoryAL.trimToSize();
      fileAL.trimToSize();
    } finally {
      // Close SFTP channel and session
      if (sftpChannel != null) {
        sftpChannel.disconnect();
      }
      if (session != null) {
        session.disconnect();
      }
    }

		directories = directoryAL.toArray(new String[directoryAL.size()]);
		files = fileAL.toArray(new String[fileAL.size()]);

		return;
	}

	@Input(required = Required.OPTIONAL)
	@Name("serverHostName")
	public void setServerHostName(String val) {
		this.serverHostName = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("serverPort")
	public void setServerPort(Long val) {
		this.serverPort = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpUsername")
	public void setSftpUsername(String val) {
		this.sftpUsername = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpPassword")
	public void setSftpPassword(String val) {
		this.sftpPassword = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("sftpServerRemoteDirectory")
	public void setSftpServerRemoteDirectory(String val) {
		this.sftpServerRemoteDirectory = val;
	}

	@Name("directories")
	public String[] getDirectories() {
		return directories;
	}

	@Name("files")
	public String[] getFiles() {
		return files;
	}

	private SmartServiceException createException(Throwable t, String key, Object... args) {
		return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
	}
}