package com.appian.ps.plugins.sftpsmartservices;

import java.util.Map;

import org.apache.commons.lang.StringUtils;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

public class SFTPConfig {
	private static final String KEY_USERNAME = "username";
	private static final String KEY_PASSWORD = "password";
	private static final String KEY_PASSPHRASE = "passphrase";
	private String serverHostname;
	private Long serverPort;
	private String username;
	private String password;
	private String passphrase;
	private String scsExternalSystemKey;
	private boolean usePerUserCredentials = false;
	private Long privateKeyDocId;
	private Long knownHostsDocId;
	private Session session;
	private ChannelSftp sftpChannel;

	public String getServerHostname() {
		return serverHostname;
	}

	public Long getServerPort() {
		return serverPort;
	}

	public String getUsername() {
		return username;
	}

	public String getPassword() {
		return password;
	}

	public String getScsExternalSystemKey() {
		return scsExternalSystemKey;
	}

	public boolean getUsePerUserCredentials() {
		return usePerUserCredentials;
	}

	public Long getPrivateKeyFileId() {
		return privateKeyDocId;
	}

	public Long getKnownHostsFileId() {
		return knownHostsDocId;
	}

	public ChannelSftp getSftpChannel() {
		return sftpChannel;
	}

	public void setServerHostname(String serverHostname) {
		this.serverHostname = serverHostname;
	}

	public void setServerPort(Long serverPort) {
		this.serverPort = serverPort;
	}

	public void setUsername(String val) {
		this.username = val;
	}

	public void setPassword(String val) {
		this.password = val;
	}

	public void setPassphrase(String val) {
		this.passphrase = val;
	}

	public void setScsExternalSystemKey(String val) {
		this.scsExternalSystemKey = val;
	}

	public void setUsePerUserCredentials(Boolean val) {
		this.usePerUserCredentials = val;
	}

	public void setPrivateKeyFileId(Long privateKeyDocId) {
		this.privateKeyDocId = privateKeyDocId;
	}

	public void setKnownHostsFileId(Long knownHostsDocId) {
		this.knownHostsDocId = knownHostsDocId;
	}

	public void connect(ContentService cs, SecureCredentialsStore scs) throws Exception {
		JSch jsch = new JSch();

		if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
			try {
				Map<String, String> credentials = usePerUserCredentials
						? scs.getUserSecuredValues(scsExternalSystemKey)
						: scs.getSystemSecuredValues(scsExternalSystemKey);

				if (!credentials.containsKey(KEY_USERNAME)) {
					throw new Exception(String.format("Required field %s does not exist in SCS (%s)", KEY_USERNAME, scsExternalSystemKey));
				}

				setUsername(credentials.get(KEY_USERNAME));
				setPassword(credentials.get(KEY_PASSWORD));
				setPassphrase(credentials.get(KEY_PASSPHRASE));
			} catch (Exception e) {
				throw new Exception(String.format("Unable to retrieve credentials from SCS: %s", e.getMessage()));
			}
		}

		// Set known hosts if defined
		if (knownHostsDocId != null && knownHostsDocId > 0) {
			String knownHostsFilePath = cs.getInternalFilename(cs.getVersionId(knownHostsDocId, ContentConstants.VERSION_CURRENT));

			jsch.setKnownHosts(knownHostsFilePath);
		}

		if (StringUtils.isEmpty(username)) {
			session = jsch.getSession(serverHostname);
		} else {
			session = jsch.getSession(username, serverHostname);
		}

		if (StringUtils.isNotEmpty(password)) {
			session.setPassword(password);
		}

		// Get private key if defined
		if (privateKeyDocId != null && privateKeyDocId > 0) {
			String privateKeyFilePath = cs.getInternalFilename(cs.getVersionId(privateKeyDocId, ContentConstants.VERSION_CURRENT));

			if (StringUtils.isEmpty(passphrase)) {
				jsch.addIdentity(privateKeyFilePath);
			} else {
				jsch.addIdentity(privateKeyFilePath, passphrase);
			}
		}

		session.setPort(serverPort.intValue());
		session.setConfig("cipher.s2c", "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");
		session.setConfig("cipher.c2s", "aes128-ctr,aes128-cbc,3des-ctr,3des-cbc,blowfish-cbc,aes192-cbc,aes256-cbc,aes256-ctr");

		if (knownHostsDocId == null || knownHostsDocId <= 0) {
			session.setConfig("StrictHostKeyChecking", "no");
		}

		session.connect();

		sftpChannel = (ChannelSftp) session.openChannel("sftp");
		sftpChannel.connect();
	}

	public void disconnect() {
		if (sftpChannel != null) {
			sftpChannel.disconnect();
		}

		if (session != null) {
			session.disconnect();
		}
	}
}