package com.appian.ps.plugins.sftpsmartservices;


import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.ConnectivityServices;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@ConnectivityServices
@Order({"scsExternalSystemKey", "usePerUserCredentials", "ServerHost", "Port", "RemoteDirPath", "Document"})
public class SendFileOverFTP extends AppianSmartService {

	private static final Logger LOG = Logger.getLogger(SendFileOverFTP.class);
	private final SmartServiceContext smartServiceCtx;
	private final SecureCredentialsStore scs;

	private String scsExternalSystemKey;
	private boolean usePerUserCredentials;	
	private String serverHost;
	private Long port;
	
	private String remoteDirPath;
	private Long document;
	private String status;
	private String error;
	private ContentService contentService;
	private final int PORT_NUMBER=21;
	private final String STATUS_FAILED ="FAILED";
	private final String STATUS_SUCCESS = "SUCCESS";

	// Secure Credentials Store variables
	private SFTPConfig config = new SFTPConfig();
	private static final String KEY_USERNAME = "username";
	private static final String KEY_PASSWORD = "password";
	private static final String NO_SCS_FIELD = "missing.scs.field";
	private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.SendFileOverFTP";	
	private String result;
	
	@Override
	public void run() throws SmartServiceException {
		// Use Secure Credential Store
	    scsExternalSystemKey = config.getScsExternalSystemKey();	    
	    usePerUserCredentials = config.getUsePerUserCredentials();
		if (StringUtils.isNotEmpty(scsExternalSystemKey)) {
			try {
				Map<String, String> credentials = usePerUserCredentials ? scs.getUserSecuredValues(scsExternalSystemKey) : scs.getSystemSecuredValues(scsExternalSystemKey);
				if (!credentials.containsKey(KEY_USERNAME)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_USERNAME, scsExternalSystemKey);
					throw new Exception(result);
				} else if (!credentials.containsKey(KEY_PASSWORD)) {
					result = getMessageForCode(NO_SCS_FIELD, KEY_PASSWORD, scsExternalSystemKey);
					throw new Exception(result);
				} 
				config.setUsername(credentials.get(KEY_USERNAME));
				config.setPassword(credentials.get(KEY_PASSWORD));
			} catch (Exception e) {
				LOG.error("Unable to retrieve credentials from SCS", e);
				this.error = e.getMessage();
				this.status = STATUS_FAILED;
				return;
			}
		}	
		
		try {
			sendFile(config.getUsername(), config.getPassword(), serverHost, port, remoteDirPath, document);
			this.status = STATUS_SUCCESS;
		} catch (Exception e) {
			LOG.error("Error sending file", e);
			this.error = e.getMessage();
			this.status= STATUS_FAILED;
		}
	}

	public SendFileOverFTP(SmartServiceContext smartServiceCtx, ContentService cs, SecureCredentialsStore scs) {
		super();
		this.scs = scs;
		this.smartServiceCtx = smartServiceCtx;
		this.contentService = cs;
	}

	/**
	 * Method that does the file transfer using PURE Java API.  
	 * @param username
	 * @param password
	 * @param host
	 * @param port
	 * @param filePath
	 * @param fileName
	 * @param documentID
	 * @throws Exception
	 */
	private void sendFile(String username, String password, String host, Long port, String filePath, Long documentID) throws Exception{
		Document d[] = (Document[])null;
		
		 try
         {
             d = contentService.download(documentID, ContentConstants.VERSION_CURRENT, Boolean.valueOf(false));
             LOG.debug("Appian document Id " + d);
         }
         catch(Exception e)
         {
             throw e;
         }
         Document ftpDoc = d[0];
         
         String fileLocation = ftpDoc.getInternalFilename();
         LOG.debug("Appian document URL " + fileLocation);
         
         //Read the file
	     	File file = new File(fileLocation);
			InputStream is = new FileInputStream(file);
			long len = file.length();
			
			if (len > Integer.MAX_VALUE)
				LOG.error("File too large");
			
			byte[] bt = new byte[(int) len];
			int offset = 0;
	        int numRead = 0;
	        while (offset < bt.length
	               && (numRead=is.read(bt, offset, bt.length-offset)) >= 0) {
	            offset += numRead;
	        }
	    
	        // Ensure all the bytes have been read in
	        if (offset < bt.length) {
	           // throw new IOException("Could not completely read file "+file.getName());
	        	LOG.error("Count not complete reading file");
	        }
	    
	        // Close the input stream and return bytes
	        is.close();
	         
         /*
	         BufferedReader isx = new BufferedReader(new FileReader(fileLocation));
		 	 int ch;
		 	 StringBuffer strContent = new StringBuffer("");
		 	 while((ch=isx.read()) != -1)
		 		strContent.append((char)ch);
		 	 isx.close();
	 	 */
 		
 		
         
		String remote_filename = (new StringBuilder(String.valueOf(d[0].getName()))).append(".").append(d[0].getExtension()).toString();
		LOG.debug("Remote filename : " + remote_filename);
		String writtenFileName = "unknown";
		
		//Check if filename needs to be over-written
		writtenFileName = d[0].getDisplayName();
				
		//check for port
		if (port==null || port.intValue()==0)
			port = new Long(PORT_NUMBER);
			
			
		String urlString = "ftp://" + username + ":" + password + "@"+host+":"+ port + filePath + "/" + writtenFileName;
		LOG.debug("URL for File transfer : " + urlString);
		URL url = new URL(urlString);
		URLConnection urlc = url.openConnection();
		LOG.debug("URL connection : " + urlc);
		
		//LOG.debug("FTP: File = " + fileLocation);
		//LOG.debug("FTP: Data: " + strContent.toString());

		OutputStream os = urlc.getOutputStream();
		//os.write(strContent.toString().getBytes());
		os.write(bt);
		os.flush();
		os.close();		
	}

	 /**
	   * Gets the corresponding message from the bundle for a given return code
	   * 
	   * @param code
	   *          the code returned from the executeTask() method
	   * @param params
	   *          the parameter values to replace
	   * @return the plain-text message
	   */
	 protected String getMessageForCode(String code, Object... params) {
	   Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
	     : smartServiceCtx.getUserLocale();
	 
	   try {
	     ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

	     return MessageFormat.format(bundle.getString(code), params);
	   } catch (Exception e) {
	     LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
	     return "";
	   }
	 }	
	 
	@Input(required = Required.ALWAYS)
	@Name("ServerHost")
	public void setServerHost(String val) {
		this.serverHost = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("Port")
	public void setPort(Long val) {
		this.port = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("scsExternalSystemKey")
	public void setExternalSystemKey(String scsExternalSystemKey) {
	   config.setScsExternalSystemKey(scsExternalSystemKey);
	}

	@Input(required = Required.ALWAYS, defaultValue = "false")
	@Name("usePerUserCredentials")
	public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
	   config.setUsePerUserCredentials(usePerUserCredentials);
	}
	
	@Input(required = Required.OPTIONAL)
	@Name("RemoteDirPath")
	public void setRemoteDirPath(String val) {
		this.remoteDirPath = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("Document")
	@DocumentDataType
	public void setDocument(Long val) {
		this.document = val;
	}

	@Name("Status")
	public String getStatus() {
		return status;
	}

	@Name("Error")
	public String getError() {
		return error;
	}

}

