// Author: Phil Russell
// Date Modified: 2010-August-19
// Appian Version: 6.0.3+

package com.appian.ps.plugins.sftpsmartservices.old;

import java.io.FileInputStream;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import com.appian.ps.plugins.sftpsmartservices.SFTPConfig;
import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.Order;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.security.external.SecureCredentialsStore;

@Deprecated
@PaletteInfo(paletteCategory="#Deprecated#", palette="#Deprecated#")
@Order({"scsExternalSystemKey", "usePerUserCredentials", "serverHostName", "serverPort", "remoteFilePath", "sourceDocument"})
public class SendFileOverSFTP extends AppianSmartService {
  private static final Logger LOG = Logger.getLogger(SendFileOverSFTP.class);
  private final SmartServiceContext smartServiceCtx;
  private final ContentService contentService;
  private final SecureCredentialsStore scs;

  // Secure Credentials Store variables
  private SFTPConfig config = new SFTPConfig();
  private static final String PLUGIN_KEY = "com.appiancorp.a2.process.runtime.activities.sftp.SendFileOverSFTPwithSCSUsernameAndPassword";	

  // Smart Service inputs
  private String remoteFilePath;
  private Long sourceDocument;

  public SendFileOverSFTP(SecureCredentialsStore scs, SmartServiceContext smartServiceCtx, ContentService cs) {
    super();
    this.scs = scs;
    this.smartServiceCtx = smartServiceCtx;
    this.contentService = cs;
  }

  private SmartServiceException createException(Throwable t, String key, Object... args) {
    return new SmartServiceException.Builder(getClass(), t).userMessage(key, args).build();
  }

  @Name("serverHostName")
  @Input(required = Required.ALWAYS)
  public void setServerHostName(String hostName) {
    config.setServerHostname(hostName);
  }

  @Name("serverPort")
  @Input(required = Required.ALWAYS, defaultValue = "22")
  public void setServerPort(Long portNumber) {
    config.setServerPort(portNumber);
  }

  @Input(required = Required.ALWAYS)
  @Name("scsExternalSystemKey")
  public void setExternalSystemKey(String scsExternalSystemKey) {
     config.setScsExternalSystemKey(scsExternalSystemKey);
  }
  
  @Input(required = Required.ALWAYS, defaultValue = "false")
  @Name("usePerUserCredentials")
  public void setUsePerUserCredentials(Boolean usePerUserCredentials) {
     config.setUsePerUserCredentials(usePerUserCredentials);
  }  
  
  @Name("remoteFilePath")
  @Input(required = Required.ALWAYS)
  public void setRemoteFilePath(String serverFilepath) {
    this.remoteFilePath = serverFilepath;
  }

  @Name("sourceDocument")
  @Input(required = Required.ALWAYS)
  @DocumentDataType
  public void setSourceDocument(Long documentId) {
    this.sourceDocument = documentId;
  }

  @Override
  public void run() throws SmartServiceException {
    String internalFilePath;
    try {
      // Obtain the current version Document object of the ACC document specified
      sourceDocument = contentService.getVersionId(sourceDocument, ContentConstants.VERSION_CURRENT);
      internalFilePath = contentService.getInternalFilename(sourceDocument).replace('\\', '/');
    } catch (Exception e) {
      LOG.error("Error getting current document", e);
      // The documentID specified does not exist in the Appian Collaboration Center
      throw createException(e, "error.doc_no_exist", e.getMessage());
    }

    try {
      // Establish a new SFTP connection using process-specific parameters
      config.connect(contentService, scs);
    } catch (Exception e) {
      LOG.error("Error connecting to SFTP server", e);
      throw createException(e, "error.sftp_conn");
    }

    try {
      // Create a copy of the Appian document on the remote server
      InputStream fileBodyIns = new FileInputStream(internalFilePath);
      config.getSftpChannel().put(fileBodyIns, remoteFilePath);

      // Close SFTP connection
      config.disconnect();
    } catch (Exception e) {
      LOG.error("Error transferring file", e);
      throw createException(e, "error.file_transfer");
    }
  }

  /**
   * Gets the corresponding message from the bundle for a given return code
   * 
   * @param code
   *          the code returned from the executeTask() method
   * @param params
   *          the parameter values to replace
   * @return the plain-text message
   */
  protected String getMessageForCode(String code, Object... params) {
    Locale locale = smartServiceCtx.getUserLocale() == null ? smartServiceCtx.getPrimaryLocale()
     : smartServiceCtx.getUserLocale();
 
    try {
      ResourceBundle bundle = ResourceBundle.getBundle(PLUGIN_KEY, locale);

      return MessageFormat.format(bundle.getString(code), params);
    } catch (Exception e) {
      LOG.error("Error getting message for code " + code + " from bundle " + PLUGIN_KEY, e);
      return "";
    }
  }    
}
